SystemOrganization addCategory: #'MemoryMentor-Tests'!
SystemOrganization addCategory: #'MemoryMentor-Visualization'!
SystemOrganization addCategory: #'MemoryMentor-Core'!

!Float class methodsFor: '*MemoryMentor-Core' stamp: 'lt 6/4/2013 18:28:54.799'!
fromIEEE64Bit: doubleWord
	"Convert the given 64 bit IEEE floating point representation into a Squeak float object (also being 64bit wide)."
	| sign newFloat |
	doubleWord negative ifTrue: [^ self error:'Cannot deal with negative numbers'].
	doubleWord = 0 ifTrue: [^ 0.0].
	sign := doubleWord bitAnd: 16r8000000000000000.
	doubleWord = sign ifTrue: [^self negativeZero].

	newFloat := self new: 2.
	newFloat basicAt: 1 put: (doubleWord bitShift: -32).
	newFloat basicAt: 2 put: (doubleWord bitAnd: 16rFFFFFFFF).
	^newFloat! !

!Float methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/4/2013 16:49:34.092'!
asIEEE64BitDoubleWord
	"Convert the receiver into a 64 bit Integer value representing the same number in IEEE 64 bit format."
	
	^ ((self basicAt: 1) bitShift: 32) + (self basicAt: 2)! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:56:15.61'!
mmBitInvert32
	^ (self bitXor: 16rFFFFFFFF) bitAnd: 16rFFFFFFFF! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:56:21.948'!
mmBitInvert64
	^ (self bitXor: 16rFFFFFFFFFFFFFFFF) bitAnd: 16rFFFFFFFFFFFFFFFF! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:56:27.416'!
mmBitInvert8
	^ (self bitXor: 16rFF) bitAnd: 16rFF! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:29:54.066'!
mmFromTwosComplement32
	| twos negative |
	negative := (self bitAt: 32) = 1.
	twos := self mmTwosComplement32.
	^ negative ifTrue: [ twos negated ] ifFalse: [ self ]! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:26:05.734'!
mmFromTwosComplement64
	| twos negative |
	negative := (self bitAt: 64) = 1.
	twos := self mmTwosComplement64.
	^ negative ifTrue: [ twos negated ] ifFalse: [ twos ]! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:56:47.142'!
mmFromTwosComplement8
	| twos negative |
	negative := (self bitAt: 8) = 1.
	twos := self mmTwosComplement8.
	^ negative ifTrue: [ twos negated ] ifFalse: [ self ]! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:22:47.056'!
mmToTwosComplement32
	self negative ifFalse: [ ^ self ].
	^ self negated mmTwosComplement32! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:26:15.034'!
mmToTwosComplement64
	self negative ifFalse: [ ^ self ].
	^ self negated mmTwosComplement64! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:57:03.27'!
mmToTwosComplement8
	self negative ifFalse: [ ^ self ].
	^ self negated mmTwosComplement8! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:20:20.028'!
mmTwosComplement32
	^ (self mmBitInvert32 + 1) bitAnd: 16rFFFFFFFF! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:26:49.662'!
mmTwosComplement64
	^ (self mmBitInvert64 + 1) bitAnd: 16rFFFFFFFFFFFFFFFF! !

!Integer methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:57:32.968'!
mmTwosComplement8
	^ (self mmBitInvert8 + 1) bitAnd: 16rFF! !

SystemWindow subclass: #MMMemoryMentor
	instanceVariableNames: 'ui uiContainer uiDescription uiMemory currentTask currentTaskInstance memoryMorph currentUser taskTree statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:39:04.052'!
changeTask: aTask
	| nextTask |
	nextTask := aTask createTask.
	nextTask ifNil: [ ^ self ].
	
	currentTask := aTask.
	currentTaskInstance := nextTask.
	
	self updateDescription.
	self updateMemoryMorph.
	ui reset visible: true.
	ui check visible: currentTaskInstance checks notEmpty! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:45:51.262'!
check
	| errors |
	currentTaskInstance ifNil: [ ^ self ].
	memoryMorph ifNil: [ ^ self ].
	
	errors := (currentTaskInstance check: memoryMorph state).
	errors ifNil: [ ^ self ]. "errors is nil means the check attempt is invalid"
	errors isEmpty
		ifTrue: [ self pass ]
		ifFalse: [ self fail: errors ].! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 11:51:04.092'!
connectSignals
	self
		connect: ui check signal: #pressed toSelector: #check;
		connect: ui reset signal: #pressed toSelector: #reset;
		connect: ui taskDescription signal: #extentChanged: toSelector: #updateDescription;
		connect: ui changeUserButton signal: #pressed toSelector: #setCurrentUser! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 10:23:02.878'!
createDescriptionText
	uiDescription := UiTextMorph new
		editView: Morph new;
 		readOnly: true;
		wrapFlag: true;
		yourself.
	ui taskDescription widget: uiDescription
		
		! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 11:36:23.264'!
createMemoryView
	uiMemory := UiContainer new
		layoutPolicy: TableLayout new;
		layoutInset: 20;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		yourself.
	ui workSpace widget: uiMemory! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:06:23.7'!
createStatisticsTable
	statistics := MMStatisticsModel new.

	ui statisticsTable
		model: statistics;
		addColumn: 'Statistic' for: (UiItemViewSimpleColumnSpec newFrom: { #text -> #name. });
		addColumn: 'Value' for: (UiItemViewSimpleColumnSpec newFrom: { #text -> #value. }).
		
	ui statisticsTable removeVerticalHeader.
		
	self updateStatisticsTable! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:40:26.456'!
createTaskTree
	taskTree := MMTaskTreeModel new.

	ui tasks
		model: taskTree;
		addColumn: (UiItemViewSimpleColumnSpec newFrom: {
			#headerText -> 'Tasks'.
			#text -> #name.
			#color -> #color.
		});
		expandAllTopLevelNodes.
		
	self updateTaskTree.
		
	self connect: ui tasks signal: #selectionChanged: toSelector: #tasksSelected:! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 10:19:38.6'!
createUi
	uiContainer := Morph new.
	uiContainer
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self addMorph: uiContainer frame: (0@0 corner: 1@1).
	ui := MMMemoryMentorUi new.
	ui setupUi: uiContainer.
	ui container addPaneSplitters.
	ui reset visible: false.
	ui check visible: false! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:40:43.8'!
currentUser: aUser
	currentUser := aUser.
	ui userLabel
		text: '<b>', (aUser ifNil: ['no user'] ifNotNil: [aUser name]), '</b>';
		color: Color white.
	self
		updateTaskTree;
		updateStatisticsTable ! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:28:33.222'!
fail: aCollectionOfErrorMessages
	currentTask isTask ifFalse: [ ^ self ].
	
	currentUser ifNotNil: [currentUser failTask: currentTask name].
	currentTask taskState: #failed.
	
	UIManager default inform: (aCollectionOfErrorMessages inject: 'You have the following errors:', String cr into: [ :errorString :nextError | errorString, String cr, nextError])! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:23:38.386'!
fillUi
	self
		createTaskTree;
		createStatisticsTable;
		createDescriptionText;
		createMemoryView;
		initializeUserLabel! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'lt 6/14/2013 16:26:58.877'!
hasYellowButtonMenu
	^ false! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:17:50.626'!
initialExtent
	^ 800@600! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:17:28.656'!
initialize
	super initialize.
	self
		model: Model new;
		setLabel: 'Memory Mentor';
		createUi;
		fillUi;
		connectSignals;
		currentUser: MMUser lastActiveUser! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 11:46:34.764'!
initializeUserLabel
	ui userLabel
		text: '<b>User</b>';
		color: Color white! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:02:56.49'!
pass
	currentTask isTask ifFalse: [ ^ self ].
	
	currentUser ifNotNil: [currentUser accomplishTask: currentTask name].
	currentTask taskState: #passed.
	UIManager default inform: 'Correct.'! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 10:55:12.598'!
requestUserName
	^ FillInTheBlankMorph
		request: 'User:'
		initialAnswer: (currentUser ifNil: [ '' ] ifNotNil: [ currentUser name ])
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:19:39.24'!
reset
	self updateMemoryMorph! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 10:57:25.77'!
setCurrentUser
	self requestUserName ifNotNil: [ :response | | user |
		user := MMUser named: response.
		self currentUser: user.
		MMUser lastActiveUser: user ]! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:14:19.944'!
tasksSelected: tasks
	tasks ifEmpty: [ ^ self ].
	tasks first isTask ifTrue: [ self changeTask: tasks first ]! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:06:39.19'!
updateDescription
	currentTaskInstance ifNil: [^ self].
	uiDescription contents: currentTaskInstance description wrappedTo: ui taskDescription width! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:19:24.43'!
updateMemoryMorph
	currentTaskInstance ifNil: [ ^ self ].
	
	uiMemory removeAllMorphs.
	currentTaskInstance memory ifNil: [ memoryMorph := nil. ^ self ].
	memoryMorph := MMMemoryMorph for: currentTaskInstance memory copy.
	uiMemory addMorph: memoryMorph! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:56:36.024'!
updateStatisticsTable
	currentUser ifNil: [^ self].
	
	statistics clear.
	currentUser statistics do: [ :statistic | statistics addStatistic: statistic ]
! !

!MMMemoryMentor methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:05:49.796'!
updateTaskTree
	currentUser ifNil: [^ self].

	(taskTree topics flatCollect: #tasks) do: [ :task | task taskState: (currentUser taskStateFor: task name) ]! !

Object subclass: #MMMemory
	instanceVariableNames: 'startAddress memory cells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:21:11.344'!
builder
	^ MMMemoryBuilder new! !

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 16:55:28.35'!
randomWithStartAddress: startAddress cellCount: aNumber
	^ (self startAddress: startAddress cellCount: aNumber)
		randomize;
		yourself! !

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:40:49.222'!
startAddress: startAddress bytes: aByteArray
	^ self 
		startAddress: startAddress 
		bytes: aByteArray 
		cellCount: (aByteArray size / 4) ceiling! !

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:41:08.974'!
startAddress: startAddress bytes: aByteArray cellCount: aNumber
	| instance |
	instance := self 
		startAddress: startAddress 
		cellCount: aNumber.
	instance 
		at: startAddress 
		putByteArray: aByteArray.
	^ instance! !

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:49:25.696'!
startAddress: startAddress cellCount: aNumber
	^ self new
		setStartAddress: startAddress cellCount: aNumber;
		yourself! !

!MMMemory class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:44:42.262'!
withStartAddress: startAddress bytes: aByteArray cellCount: aNumber
	self deprecated: 'use #startAddress:bytes:cellCount: instead'.
	^ self 
		startAddress: startAddress 
		bytes: aByteArray 
		cellCount: aNumber! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 10:32:29.536'!
= anotherMMMemory
	self class == anotherMMMemory class ifFalse: [ ^ false ].
	startAddress == anotherMMMemory startAddress ifFalse: [ ^ false ].
	
	^ memory = anotherMMMemory memory! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 6/25/2013 13:14:59.752'!
at: address putByte: byte
	byte > 16rFF ifTrue: [ self error: 'Byte value out of range' ].
	memory byteAt: (self indexForAddress: address) put: byte! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:46:59.024'!
at: address putByteArray: byteArray
	| fittingByteArray |
	fittingByteArray := byteArray size > (cells size * 4) 
		ifTrue: [ byteArray first: cells size * 4 ] 
		ifFalse: [ byteArray ].
	fittingByteArray
		withIndexDo: [ :byte :index |
		self at: address+index-1 putByte: (byteArray at: index)]! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 6/25/2013 13:16:27.518'!
at: address putChar: char
	(char < -127 or: [ char > 128 ]) ifTrue: [ self error: 'Character value out of range' ].
	memory charAt: (self indexForAddress: address) put: char! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:55:22.043'!
at: address putDouble: double
	memory doubleAt: (self indexForAddress: address) put: double! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:54:31.805'!
at: address putFloat: aFloat
	memory floatAt: (self indexForAddress: address) put: aFloat! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 15:32:11.56'!
at: address putInt: int
	memory intAt: (self indexForAddress: address) put: int! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:55:02.225'!
at: address putLong: long
	memory longAt: (self indexForAddress: address) put: long! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:56:41.785'!
at: address putUChar: unsignedChar
	memory ucharAt: (self indexForAddress: address) put: unsignedChar! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:55:52.034'!
at: address putUInt: unsignedInt
	memory uintAt: (self indexForAddress: address) put: unsignedInt! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 17:40:08.552'!
at: address putULong: unsignedLong
	memory ulongAt: (self indexForAddress: address) put: unsignedLong! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:20.703'!
byteAt: address
	 ^ memory at: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 5/3/2013 13:04:27.336'!
byteSize
	^ memory size! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 14:14:26.12'!
cellAtAddress: address
	^ cells at: (self cellIndexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 5/8/2013 11:53:43.905'!
cellCount
	^ cells size! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 14:13:45.782'!
cellIndexForAddress: address
	^ ((address roundDownTo: 4) - startAddress)/4 + 1! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 13:24:07.742'!
cellsDo: aBlock
	^ cells do: aBlock! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:25.854'!
charAt: address
	 ^ memory charAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:37:46.814'!
copy
	^ self class
		startAddress: startAddress
		bytes: memory copy! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:33.25'!
doubleAt: address
	 ^ memory doubleAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 5/8/2013 11:54:00.067'!
endAddress
	^ startAddress + (self cellCount * MMMemoryCell size)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:29.099'!
floatAt: address
	 ^ memory floatAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 11:44:58.182'!
hash
	^ self startAddress hash bitXor: self memory hash! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 14:12:46.656'!
indexForAddress: address
	^ address - startAddress + 1! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 15:32:31.124'!
intAt: address
	 ^ memory intAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:16.989'!
longAt: address
	 ^ memory longAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 10:32:06.59'!
memory
	^ memory! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 10:54:46.198'!
randomize
	| random |
	random := Random new.
	1 to: memory size do: [ :i | memory byteAt: i put: (random nextInt: 255) - 1 ].! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:06:02.714'!
setStartAddress: address cellCount: aNumber
	"self assert: [ (address roundUpTo: 4) == address ]."
	
	startAddress := address.
	memory := ByteArray new: aNumber*4.
	cells := OrderedCollection new.
	1 to: aNumber do: [ :i |
		cells add: (MMMemoryCell at: (4*(i-1) + startAddress) in: self)
	]! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 14:12:09.978'!
startAddress
	^ startAddress! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:37.344'!
ucharAt: address
	 ^ memory ucharAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:49:52.128'!
uintAt: address
	 ^ memory uintAt: (self indexForAddress: address)! !

!MMMemory methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:50:41.463'!
ulongAt: address
	 ^ memory ulongAt: (self indexForAddress: address)! !

Object subclass: #MMMemoryBuilder
	instanceVariableNames: 'fillCommands currentOffset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 10:51:39.452'!
addAlign: anInteger
	self alignAddressTo: anInteger! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:33:27.514'!
addByteArray: aByteArray
	self addFillCommand: #at:putByteArray: value: aByteArray.
	self addOffset: aByteArray size! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:07:20.074'!
addCString: aString
	self addString: aString, (Character value: 0) asString! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:33:46.256'!
addChar: anInteger
	self addFillCommand: #at:putChar: value: anInteger.
	self addOffset: 1! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:52:34.078'!
addDouble: aFloat
	self alignAddressTo: 8.
	self addFillCommand: #at:putDouble: value: aFloat.
	self addOffset: 8! !

!MMMemoryBuilder methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:33:07.03'!
addFillCommand: aSymbol value: aValue
	fillCommands add: { aSymbol. currentOffset. aValue }! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:34:17.746'!
addFloat: aFloat
	self alignAddressTo: 4.
	self addFillCommand: #at:putFloat: value: aFloat.
	self addOffset: 4! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:34:33.838'!
addInt: anInteger
	self alignAddressTo: 4.
	self addFillCommand: #at:putInt: value: anInteger.
	self addOffset: 4! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:52:40.98'!
addLong: anInteger
	self alignAddressTo: 8.
	self addFillCommand: #at:putLong: value: anInteger.
	self addOffset: 8! !

!MMMemoryBuilder methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:23:45.41'!
addOffset: aNumber
	currentOffset := currentOffset + aNumber! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 10:49:05.242'!
addPadding: byteSize
	self addOffset: byteSize! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 10:55:20.84'!
addRandom: byteSize
	| random |
	random := Random new.
	self addByteArray: ((1 to: byteSize) collect: [ :i | (random nextInt: 256) - 1 ])! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 10:52:21.85'!
addString: aString
	self addByteArray: aString asByteArray! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:52:52.966'!
addStruct: aStruct
	self alignAddressTo: 8.
	aStruct do: [ :assoc | self addValue: assoc value ofType: assoc key ].
	self alignAddressTo: 8! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:35:03.396'!
addUChar: anInteger
	self addFillCommand: #at:putUChar: value: anInteger.
	self addOffset: 1! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:35:15.934'!
addUInt: anInteger
	self alignAddressTo: 4.
	self addFillCommand: #at:putUInt: value: anInteger.
	self addOffset: 4! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:52:46.558'!
addULong: anInteger
	self alignAddressTo: 8.
	self addFillCommand: #at:putULong: value: anInteger.
	self addOffset: 8! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:14:59.254'!
addValue: value ofType: type
	type caseOf: {
		"[ #byteArray ] -> [ self addByteArray: value ]."
		"[ #cstring ] -> [ self addCString: value ]."
		[ #char ] -> [ self addChar: value ].
		[ #double ] -> [ self addDouble: value ].
		[ #float ] -> [ self addFloat: value ].
		[ #int ] -> [ self addInt: value ].
		[ #long ] -> [ self addLong: value ].
		[ #uchar ] -> [ self addUChar: value ].
		[ #uint ] -> [ self addUInt: value ].
		[ #ulong ] -> [ self addULong: value ].
		[ #struct ] -> [ self addStruct: value ].
	}! !

!MMMemoryBuilder methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:24:10.438'!
alignAddressTo: aNumber
	currentOffset := currentOffset roundUpTo: aNumber! !

!MMMemoryBuilder methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:29:16.448'!
alignedMemorySize
	^ (currentOffset roundUpTo: 4) / 4! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:22:47.904'!
build
	^ self build: 0! !

!MMMemoryBuilder methodsFor: 'dsl' stamp: 'WS 7/8/2013 11:37:46.99'!
build: anAddress
	| memory |
	memory := MMMemory startAddress: anAddress cellCount: self alignedMemorySize.
	
	fillCommands do: [ :fillCommand | | selector offset value |
		selector := fillCommand first.
		offset := fillCommand second.
		value := fillCommand third.
		
		memory perform: selector with: anAddress + offset with: value.
	].
	
	^ memory! !

!MMMemoryBuilder methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:35:44.29'!
initialize
	super initialize.
	
	fillCommands := OrderedCollection new.
	currentOffset := 0! !

Object subclass: #MMMemoryCell
	instanceVariableNames: 'address memory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMMemoryCell class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:38:27.903'!
at: address in: memory
	 ^ self new
		setAddress: address memory: memory;
		yourself! !

!MMMemoryCell class methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 12:41:51.122'!
size
	 ^ 4! !

!MMMemoryCell methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 14:15:11.996'!
address
	^ address! !

!MMMemoryCell methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 15:22:27.266'!
byteAt: index
	^ memory byteAt: address + index! !

!MMMemoryCell methodsFor: 'as yet unclassified' stamp: 'lt 5/8/2013 11:53:00.459'!
memory
	^ memory! !

!MMMemoryCell methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:38:42.173'!
setAddress: anAddress memory: aMemory
	address := anAddress.
	memory := aMemory! !

Object subclass: #MMMemoryState
	instanceVariableNames: 'memoryMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMMemoryState class methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:05:49.508'!
for: aMemoryMorph
	^ self new
		memoryMorph: aMemoryMorph;
		yourself! !

!MMMemoryState methodsFor: 'checking' stamp: 'WS 6/29/2013 21:39:48.118'!
annotationAt: anAddress
	^ (memoryMorph groupAtAddress: anAddress)
		ifNil: [ '' ]
		ifNotNil: [ :group | (group valueMorphAtAddress: anAddress)
			ifNil: [ '' ]
			ifNotNil: [ :valueMorph | valueMorph annotation ]
		]! !

!MMMemoryState methodsFor: 'checking' stamp: 'WS 6/27/2013 11:29:52.292'!
byteAt: anAddress
	^ memoryMorph memory byteAt: anAddress! !

!MMMemoryState methodsFor: 'checking' stamp: 'RL 6/26/2013 17:15:55.35'!
hasPointerArrowFrom: anAddress to: anotherAddress
	^ (memoryMorph pointerArrows
		detect: [ :arrow | arrow fromCell notNil and: [ arrow fromCell address = anAddress ] and: [ arrow toCell notNil ] and: [ arrow toCell address = anotherAddress ] ]
		ifNone: [ nil ]) notNil! !

!MMMemoryState methodsFor: 'checking' stamp: 'lt 7/1/2013 16:31:10.456'!
hasPointerFrom: anAddress to: anotherAddress
	^ (self typeAt: anAddress) = #pointer 
		and: [(self memory ulongAt: anAddress) = anotherAddress]! !

!MMMemoryState methodsFor: 'checking' stamp: 'RL 6/26/2013 17:07:17.004'!
memory
	^ memoryMorph memory! !

!MMMemoryState methodsFor: 'accessing' stamp: 'RL 6/26/2013 17:06:27.822'!
memoryMorph: anObject
	memoryMorph := anObject! !

!MMMemoryState methodsFor: 'checking' stamp: 'RL 6/26/2013 17:10:46.068'!
typeAt: anAddress
	^ (memoryMorph groupAtAddress: anAddress)
		ifNil: [ #none ]
		ifNotNil: [ :group | group type ]! !

Object subclass: #MMPointerArrow
	instanceVariableNames: 'arrow fromCell toCell'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMPointerArrow class methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 14:03:35.058'!
from: aMemoryCell
	^ self
		from: aMemoryCell
		to: nil! !

!MMPointerArrow class methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 14:03:40.86'!
from: aMemoryCell to: anotherMemoryCell
	^ super new
		fromCell: aMemoryCell;
		toCell: anotherMemoryCell;
		yourself! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 13:12:27.58'!
arrow
	^ arrow! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:41:06.194'!
attachmentPointToCellGroupMorph: aCellGroupMorph
	^ aCellGroupMorph rightCenterInWorld! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:29:48.178'!
cellGroupMorphAtPosition: position
	^ (self currentWorld morphsAt: position)
		detect: [ :morph | morph isKindOf: MMCellGroupMorph ]
		ifNone: [ nil ]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 14:02:26.484'!
fromCell
	^ fromCell! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 14:18:46.868'!
fromCell: aMemoryCell
	fromCell ifNotNil: [ fromCell pointerArrow: nil ].
	fromCell := aMemoryCell.
	
	fromCell ifNotNil: [
		fromCell pointerArrow: self.
		arrow := MMRightAngledArrow from: (self relativePointToPointerArea: (self attachmentPointToCellGroupMorph: fromCell)) to: (self relativePointToPointerArea: (self positionSnappedToCellGroupMorph: self currentWorld currentHand center)) spacing: 10 + aMemoryCell memoryMorph arrowCount * 7.
		aMemoryCell memoryMorph pointerArea addArrow: arrow
	]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 12:05:00.136'!
handleListenEvent: evt
	toCell ifNil: [
		self reinitializeArrowFrom: (self attachmentPointToCellGroupMorph: fromCell) to: (self positionSnappedToCellGroupMorph: evt position).
	].
	
	(evt isMouseUp) ifTrue: [ 
		self currentWorld currentHand removeMouseListener: self.
		
		(self cellGroupMorphAtPosition: evt position) ifNil: [
			fromCell removePointerArrow.
		] ifNotNil: [ :cellGroupMorph |
			self toCell: cellGroupMorph.
		]
	]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 13:48:45.9'!
initialize
	super initialize.
	
	self currentWorld currentHand addMouseListener: self! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:39:30.192'!
positionSnappedToCellGroupMorph: position
	^ (self cellGroupMorphAtPosition: position)
		ifNil: [ position ]
		ifNotNil: [ :cellGroupMorph | self attachmentPointToCellGroupMorph: cellGroupMorph ]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 12:03:52.406'!
reinitializeArrowFrom: position to: anotherPosition
	arrow ifNotNil: [
		arrow initializeFrom: (self relativePointToPointerArea: position) to: (self relativePointToPointerArea: anotherPosition)
	]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:33:05.12'!
relativePointToPointerArea: position
	^ fromCell
		ifNil: [ ^ 0@0 ]
		ifNotNil: [ position - fromCell memoryMorph pointerArea positionInWorld ]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 12:00:22.968'!
remove
	arrow delete.
	fromCell ifNotNil: [ fromCell memoryMorph pointerArea removeArrow: arrow]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 14:02:47.636'!
toCell
	^ toCell! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 16:05:32.838'!
toCell: aMemoryCell
	toCell := aMemoryCell.
	
	toCell ifNotNil: [
		self currentWorld currentHand removeMouseListener: self.
		
		self updateArrow.
		
		fromCell retargetedPointer: toCell
	]! !

!MMPointerArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 16:06:17.358'!
updateArrow
	(fromCell isNil or: [ toCell isNil ]) ifTrue: [ ^ self ].
	self reinitializeArrowFrom: (self attachmentPointToCellGroupMorph: fromCell) to: (self attachmentPointToCellGroupMorph: toCell)! !

Object subclass: #MMStatisticsModel
	uses: TUiItemModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

MMStatisticsModel class
	uses: TUiItemModel classTrait
	instanceVariableNames: ''!

!MMStatisticsModel methodsFor: 'item model - signals' stamp: 'RL 7/1/2013 13:42:56.39'!
addStatistic: aStatistic
	data add: aStatistic.
	self childInserted: data size inGroup: #data! !

!MMStatisticsModel methodsFor: 'item model - signals' stamp: 'RL 7/1/2013 13:44:10.914'!
clear
	| size |
	size := data size.
	data removeAll.
	self childrenRemovedFrom: 1 to: size inGroup: #data
	! !

!MMStatisticsModel methodsFor: 'item model - hierarchy' stamp: 'RL 7/1/2013 13:24:11.896'!
data
	^ data! !

!MMStatisticsModel methodsFor: 'item model - properties' stamp: 'RL 7/1/2013 13:24:24.552'!
groups
	^ #(data)! !

!MMStatisticsModel methodsFor: 'item model - signals' stamp: 'RL 7/1/2013 13:41:45.908'!
initialize 
	super initialize.
	data := OrderedCollection new! !

Object subclass: #MMTask
	instanceVariableNames: 'description memory checks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

MMTask class
	instanceVariableNames: 'random'!

MMTask class
	instanceVariableNames: 'random'!

!MMTask class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:56:17.154'!
description: aString memory: aMemory checks: aCollectionOfChecks
	^ self new
		description: aString;
		memory: aMemory;
		checks: aCollectionOfChecks;
		yourself! !

!MMTask class methodsFor: 'pointers' stamp: 'WS 7/8/2013 14:44:01.934'!
drawPointerArrow
	| memory int startAddress intAddress pointerAddress |
	startAddress := (self randomBetween: 10 and: 20) * 4.
	memory := MMMemory randomWithStartAddress: startAddress cellCount: 20.
	intAddress := (self randomBetween: startAddress and: startAddress + 9) roundUpTo: 4.
	pointerAddress := (self randomBetween: startAddress + 10 and: startAddress + 19) roundUpTo: 4.
	int := self randomBetween: 1 and: 65000.
	
	memory at: intAddress putInt: int.
	memory at: pointerAddress putULong: intAddress.
	
	^ self
		description: ('To successfully complete this challenge, manipulate the cell at address 0x{1} to be a pointer that refers to the value at 0x{2}, which is an integer. 

Then, retrieve the target''s value in decimal representation and answer it when prompted after clicking ''check''.'
			format: {pointerAddress printStringBase: 16. intAddress printStringBase: 16.})
		memory: memory
		checks: {
			[ :memoryState | self requestInt ifNotNil: [ :answeredInt | answeredInt = int ] ] -> 'The requested integer isn''t correct.'.
			[ :memoryState | memoryState hasPointerFrom: pointerAddress to: intAddress ] -> 'The pointer is not set correctly.'
		}! !

!MMTask class methodsFor: 'pointers' stamp: 'WS 7/8/2013 14:43:40.5'!
integerEndianness
	^ self
		description: 'The memory should have an integer value at address 0x78.

Change the integer''s value so that the first 8 bits in memory are zero, then there are 8 bits set and the last 16 bits are zero.
To complete this task, answer with the value of the integer at address 0x78.'
		memory: (MMMemory randomWithStartAddress: 16r70 cellCount: 5)
		checks: {
			[ :memoryState | (memoryState byteAt: 16r78) = 0 ] -> 'Byte at 0x78 isn''t 0'.
			[ :memoryState | (memoryState byteAt: 16r79) = 255 ] -> 'Byte at 0x79 isn''t 255'.
			[ :memoryState | (memoryState byteAt: 16r7A) = 0 ] -> 'Byte at 0x7A isn''t 0'.
			[ :memoryState | (memoryState byteAt: 16r7B) = 0 ] -> 'Byte at 0x7B isn''t 0'.
			[ :memoryState | self requestInt ifNotNil: [ :int | int = 65280 ] ] -> 'The integer value isn''t correct.'.
		}! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 13:57:56.554'!
manualInstanceInitialization
	^ self
		description: ''
		memory: MMMemory new
		checks: {}! !

!MMTask class methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 13:21:29.946'!
new
	^ super new initialize ! !

!MMTask class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 13:26:32.552'!
random
	^ random ifNil: [ random := Random new ]! !

!MMTask class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 13:28:52.676'!
randomBetween: start and: end
	^ start + (self random nextInt: end - start + 1) - 1! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:38:00.828'!
randomMemory
	| byteSize |
	byteSize := (self requestInt: 'Byte size of memory' default: 100) ifNil: [ ^ nil ].
	byteSize strictlyPositive ifFalse: [ self error: 'No valid memory size' ].
	^ self
		description: self standardDescription
		memory: (MMMemory randomWithStartAddress: 16r0 cellCount: (byteSize roundUpTo: 4) / 4)
		checks: {}! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:29:27.442'!
requestInt
	^ self requestInt: 'Answer:'! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:29:15.118'!
requestInt: aTitle
	^ self requestInt: aTitle default: 0! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:28:51.648'!
requestInt: aTitle default: aNumber
	| answer |
	answer := FillInTheBlankMorph
		request: aTitle
		initialAnswer: aNumber printString
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true.
	^ answer ifNotNil: [ [ Integer readFrom: answer ] on: Error do: [ ^ nil ] ]! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:08:45.022'!
standardDescription
	^ 'Hello!!

In the rightmost pane you find a representation of a part of main memory. The memory excerpt is four byte aligned. That is, your see and address followed by the four bytes starting at that address.

You can interact with the memory representation through a context menu. To open the context menu, right click on any cell you want to manipulate. The context menu is separated into three parts. The first part operates on the cell you opened the menu on. The second part operates on four byte aligned groups, while the third part of the context menu changes the view for the complete memory representation. 

Initially, you see raw bits in hexadecimal representation. To see raw bits in binary representation, open the context menu, choose ''Show raw bits as...'', and select ''Binary'' instead.

If you want to see a certain aligned address interpreted using a specific type, open the context menu, choose ''Interpret memory as...'', then select an interpretation as well as a representation.

Interpreting a specific address as a pointer adds an arrow to the address the pointer refers to, if the target address is visible in memory. 
To interpret an address as a pointer and visually set the pointer''s target address, choose ''Retarget pointer'', then click to the address the pointer should refer to.

To change the value at an address in the current interpretation (e.g. integer) and the current representation (e.g. decimal), open the content menu, choose ''Change value'', enter the new value, and accept the change. If the current interpretation or the current representation is not appropriate for the new value, you have to change the interpretation and representation before entering it.

It is also possible to see the raw bits together with their interpretation. Just open the context menu and choose either the ''Show bits'' or the ''Show bits for all'' entry, depending on whether you want to see bits for just the cell you clicked on or for all cells. The menu options ''Show bits'' and ''Hide bits for all'' subsequently allow to hide bits.

Every cell can also be annotated with arbitrary text. Open the context menu, choose ''Annotate'', enter your additional information, and accept.

Have fun!!'! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 13:58:17.196'!
structFields
	^ self
		description: 'The memory at address 0x1A should be interpreted using the following unpacked struct.

struct Employee {
	unsigned int age;
	const char* name;
	Address* address;
	unsigned int salary;
};

Change the cells of this struct to their respective types. Then, annotate the cells with their variable names: ''age'', ''name'', ''address'' and ''salary'''
		memory: (MMMemory startAddress: 16r16 bytes: #[0 0 0 0 38 0 0 0 50 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 232 253 0 0 84 104 111 109 97 115 32 77 101 105 101 114 0 0 0 0])
		checks: {
			[ :memoryState | (memoryState annotationAt: 16r1A) = 'age' ] -> 'Annotation of ''age'' isn''t correct.'.
			[ :memoryState | (memoryState annotationAt: 16r1E) = 'name' ] -> 'Annotation of ''name'' isn''t correct.'.
			[ :memoryState | (memoryState annotationAt: 16r26) = 'address' ] -> 'Annotation of ''address'' isn''t correct.'.
			[ :memoryState | (memoryState annotationAt: 16r2E) = 'salary' ] -> 'Annotation of ''salary'' isn''t correct.'.
			[ :memoryState | (memoryState typeAt: 16r1A) = #uint ] -> 'Type at 0x1A isn''t an unsigned int.'.
			[ :memoryState | (memoryState typeAt: 16r1E) = #pointer ] -> 'Type at 0x1E isn''t a pointer.'.
			[ :memoryState | (memoryState typeAt: 16r26) = #pointer ] -> 'Type at 0x26 isn''t a pointer.'.
			[ :memoryState | (memoryState typeAt: 16r2E) = #uint ] -> 'Type at 0x2E isn''t an unsigned int.'
		}! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:42:54.254'!
valueOfFieldInStruct
	^ self
		description: 'The memory at 0x1A should be interpreted using the following unpacked struct.

struct Employee {
	const char* name;
	unsigned char age;
	Date startDate;
	unsigned int salary;
};

struct Date {
	unsigned int year;
	unsigned char month;
	unsigned char dayOfMonth;
};

Answer with the salary of the employee at address 0x1A.'
		memory: (MMMemory startAddress: 16r16 bytes: #[0 0 0 0 50 0 0 0 0 0 0 0 38 0 0 0 211 7 0 0 5 15 0 0 94 151 0 0 84 104 111 109 97 115 32 77 101 105 101 114 0 0 0 0])
		checks: {
			[ :memoryState | self requestInt ifNotNil: [ :int | int = 38750 ] ] -> 'Requested integer isn''t correct.'
		}! !

!MMTask class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:37:53.64'!
zeroMemory
	| byteSize |
	byteSize := (self requestInt: 'Byte size of memory' default: 100) ifNil: [ ^ nil ].
	byteSize strictlyPositive ifFalse: [ self error: 'No valid memory size' ].
	^ self
		description: self standardDescription
		memory: (MMMemory startAddress: 16r0 cellCount: (byteSize roundUpTo: 4) / 4)
		checks: {}! !

!MMTask methodsFor: 'accessing' stamp: 'WS 7/8/2013 14:45:04.284'!
check: aState
	| errors |
	errors := OrderedCollection new.
	
	checks do: [ :each | | evaluation |
		evaluation := each key value: aState.
		evaluation ifNil: [ ^ nil ].
		evaluation = true ifFalse: [ errors add: each value ]].
	
	^ errors! !

!MMTask methodsFor: 'accessing' stamp: 'WS 7/1/2013 10:40:14.058'!
checks
	^ checks! !

!MMTask methodsFor: 'accessing' stamp: 'WS 7/1/2013 10:40:36.846'!
checks: aCollection
	checks := aCollection! !

!MMTask methodsFor: 'accessing' stamp: 'RL 6/14/2013 13:17:50.928'!
description
	^ description! !

!MMTask methodsFor: 'accessing' stamp: 'RL 6/14/2013 13:17:50.938'!
description: anObject
	description := anObject! !

!MMTask methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:58:53.64'!
initialize
	description := ''.
	checks := OrderedCollection new.! !

!MMTask methodsFor: 'accessing' stamp: 'RL 6/14/2013 13:18:10.854'!
memory
	^ memory! !

!MMTask methodsFor: 'accessing' stamp: 'RL 6/14/2013 13:18:10.868'!
memory: anObject
	memory := anObject! !

Object subclass: #MMTaskTreeModel
	uses: TUiItemModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

MMTaskTreeModel class
	uses: TUiItemModel classTrait
	instanceVariableNames: ''!

!MMTaskTreeModel methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:15:49.606'!
createHierarchy
	^ {
		MMGameMode freePlay.
		MMGameMode lessons
	}! !

!MMTaskTreeModel methodsFor: 'item model - hierarchy' stamp: 'RL 6/5/2013 13:45:33.44'!
data
	^ data! !

!MMTaskTreeModel methodsFor: 'item model - hierarchy' stamp: 'RL 6/5/2013 13:40:04.898'!
groups
	^ #(data)! !

!MMTaskTreeModel methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 14:16:28.518'!
initialize 
	super initialize.
	data := self createHierarchy! !

!MMTaskTreeModel methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:47:45.89'!
topics
	^ self data flatCollect: #topics! !

Object subclass: #MMUser
	instanceVariableNames: 'name taskStates statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

MMUser class
	instanceVariableNames: 'Users LastActiveUser'!

MMUser class
	instanceVariableNames: 'Users LastActiveUser'!

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 12:50:14.866'!
lastActiveUser
	^ LastActiveUser! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 12:49:01.362'!
lastActiveUser: aUser
	LastActiveUser := aUser! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'RL 6/27/2013 12:41:34.562'!
named: aString
	^ self users 
		at: aString 
		ifAbsentPut: (MMUser new name: aString; yourself)! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 12:49:50.307'!
reset
	self 
		resetUsers;
		resetLastActiveUser! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 12:50:04.938'!
resetLastActiveUser
	self lastActiveUser: nil! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 11:21:11.123'!
resetUsers
	Users := nil! !

!MMUser class methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 11:20:49.054'!
users
	^ Users ifNil: [Users := Dictionary new]! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 14:01:52.302'!
accomplishTask: aString
	taskStates at: aString put: #passed.
	self
		increaseStatistic: #attempts;
		increaseStatistic: #passes! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 14:01:59.444'!
failTask: aString
	taskStates at: aString put: #failed.
	self
		increaseStatistic: #attempts;
		increaseStatistic: #failures! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 14:00:34.294'!
increaseStatistic: aSymbol
	| statistic |
	statistic := statistics at: aSymbol.
	statistic value: (statistic value + 1)! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:59:09.046'!
initialize 
	taskStates := Dictionary new.
	statistics := Dictionary newFrom: {
		#attempts -> (MMStatistic named: 'Attempts').
		#passes -> (MMStatistic named: 'Passes').
		#failures -> (MMStatistic named: 'Failures').
	}.! !

!MMUser methodsFor: 'accessing' stamp: 'lt 6/27/2013 11:34:46.699'!
name
	^ name! !

!MMUser methodsFor: 'accessing' stamp: 'lt 6/27/2013 11:34:08.928'!
name: anObject
	name := anObject! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 12:55:26.421'!
printOn: aStream
	super printOn: aStream.
	
	aStream 
		nextPutAll: ' (';
		print: self name;
		nextPutAll: ')'! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:59:45.36'!
setStatistic: aSymbol to: aValue
	(statistics at: aSymbol) value: aValue! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:55:55.464'!
statistics
	^ statistics values! !

!MMUser methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 10:48:20.796'!
taskStateFor: aString
	^ taskStates at: aString ifAbsentPut: [ #unattempted ]! !

UserInterface subclass: #MMMemoryMentorUi
	instanceVariableNames: 'container userLabel changeUserButton statisticsTable workSpace taskDescription check reset tasks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.474'!
changeUserButton
	"No description given."
	
	^ changeUserButton! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.496'!
check
	"No description given."
	
	^ check! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.46'!
container
	"No description given."
	
	^ container! !

!MMMemoryMentorUi methodsFor: 'ui - private' stamp: 'RL 7/1/2013 13:06:46.556'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	| unnamedMorph1 unnamedMorph2 unnamedMorph3 unnamedMorph4 unnamedMorph5 unnamedMorph6 |
	
	panel := UiContainer new.
	panel changeProportionalLayout.
	panel extent: (546@424).
	panel position: (0@0).
	panel hResizing: (#spaceFill).
	panel vResizing: (#spaceFill).
	panel minWidth: (2).
	panel minHeight: (2).
	panel cellInset: (4).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (2).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	panel color: (Color transparent).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color transparent).
	panel setProperty: #objectName toValue: 'panel'.
	container := UiContainer new.
	container changeProportionalLayout.
	container extent: (542@420).
	container position: (2@2).
	container hResizing: (#spaceFill).
	container vResizing: (#spaceFill).
	container minWidth: (2).
	container minHeight: (2).
	container cellInset: (4).
	container cellPositioning: (#center).
	container cellSpacing: (#none).
	container layoutInset: (0).
	container listCentering: (#topLeft).
	container listDirection: (#topToBottom).
	container listSpacing: (#none).
	container maxCellSize: (1073741823).
	container minCellSize: (0).
	container wrapCentering: (#topLeft).
	container wrapDirection: (#none).
	container frameFractions: (0@0 corner: 1@1).
	container frameOffsets: (0@0 corner: 0@0).
	container balloonText: (nil).
	container balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.929)).
	container color: (Color transparent).
	container borderWidth: (0).
	container borderStyle2: (#simple).
	container borderColor: (Color black).
	container setProperty: #objectName toValue: 'container'.
	unnamedMorph1 := UiContainer new.
	unnamedMorph1 changeTableLayout.
	unnamedMorph1 extent: (108@105).
	unnamedMorph1 position: (2@2).
	unnamedMorph1 hResizing: (#spaceFill).
	unnamedMorph1 vResizing: (#spaceFill).
	unnamedMorph1 minWidth: (2).
	unnamedMorph1 minHeight: (2).
	unnamedMorph1 cellInset: (4).
	unnamedMorph1 cellPositioning: (#center).
	unnamedMorph1 cellSpacing: (#none).
	unnamedMorph1 layoutInset: (4).
	unnamedMorph1 listCentering: (#topLeft).
	unnamedMorph1 listDirection: (#topToBottom).
	unnamedMorph1 listSpacing: (#none).
	unnamedMorph1 maxCellSize: (1073741823).
	unnamedMorph1 minCellSize: (0).
	unnamedMorph1 wrapCentering: (#topLeft).
	unnamedMorph1 wrapDirection: (#none).
	unnamedMorph1 frameFractions: (0@0 corner: 0.2@0.25).
	unnamedMorph1 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph1 balloonText: (nil).
	unnamedMorph1 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.937)).
	unnamedMorph1 color: (Color transparent).
	unnamedMorph1 borderWidth: (0).
	unnamedMorph1 borderStyle2: (#simple).
	unnamedMorph1 borderColor: (Color black).
	unnamedMorph2 := UiContainer new.
	unnamedMorph2 changeTableLayout.
	unnamedMorph2 extent: (100@30).
	unnamedMorph2 position: (6@6).
	unnamedMorph2 hResizing: (#spaceFill).
	unnamedMorph2 vResizing: (#shrinkWrap).
	unnamedMorph2 minWidth: (2).
	unnamedMorph2 minHeight: (2).
	unnamedMorph2 cellInset: (4).
	unnamedMorph2 cellPositioning: (#center).
	unnamedMorph2 cellSpacing: (#none).
	unnamedMorph2 layoutInset: (4).
	unnamedMorph2 listCentering: (#topLeft).
	unnamedMorph2 listDirection: (#leftToRight).
	unnamedMorph2 listSpacing: (#none).
	unnamedMorph2 maxCellSize: (1073741823).
	unnamedMorph2 minCellSize: (0).
	unnamedMorph2 wrapCentering: (#topLeft).
	unnamedMorph2 wrapDirection: (#none).
	unnamedMorph2 frameFractions: (0@0 corner: 1@1).
	unnamedMorph2 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph2 balloonText: (nil).
	unnamedMorph2 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.941)).
	unnamedMorph2 color: (Color transparent).
	unnamedMorph2 borderWidth: (0).
	unnamedMorph2 borderStyle2: (#simple).
	unnamedMorph2 borderColor: (Color black).
	userLabel := UiLabel new.
	userLabel changeTableLayout.
	userLabel extent: (27@19).
	userLabel position: (10@12).
	userLabel hResizing: (#shrinkWrap).
	userLabel vResizing: (#shrinkWrap).
	userLabel minWidth: (2).
	userLabel minHeight: (2).
	userLabel cellInset: (0).
	userLabel cellPositioning: (#center).
	userLabel cellSpacing: (#none).
	userLabel layoutInset: (0).
	userLabel listCentering: (#topLeft).
	userLabel listDirection: (#leftToRight).
	userLabel listSpacing: (#none).
	userLabel maxCellSize: (1073741823).
	userLabel minCellSize: (0).
	userLabel wrapCentering: (#topLeft).
	userLabel wrapDirection: (#none).
	userLabel frameFractions: (0@0 corner: 1@1).
	userLabel frameOffsets: (0@0 corner: 0@0).
	userLabel balloonText: (nil).
	userLabel balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.933)).
	userLabel text: ('User').
	userLabel setProperty: #formByDesigner toValue: 'nil'.
	userLabel form: (nil).
	userLabel setProperty: #objectName toValue: 'userLabel'.
	unnamedMorph2 addChild: userLabel.
	unnamedMorph3 := UiSpacer new.
	unnamedMorph3 extent: (5@20).
	unnamedMorph3 position: (41@11).
	unnamedMorph3 hResizing: (#spaceFill).
	unnamedMorph3 vResizing: (#rigid).
	unnamedMorph3 minWidth: (2).
	unnamedMorph3 minHeight: (2).
	unnamedMorph3 cellInset: (0).
	unnamedMorph3 cellPositioning: (#center).
	unnamedMorph3 cellSpacing: (#none).
	unnamedMorph3 layoutInset: (0).
	unnamedMorph3 listCentering: (#topLeft).
	unnamedMorph3 listDirection: (#topToBottom).
	unnamedMorph3 listSpacing: (#none).
	unnamedMorph3 maxCellSize: (1073741823).
	unnamedMorph3 minCellSize: (0).
	unnamedMorph3 wrapCentering: (#topLeft).
	unnamedMorph3 wrapDirection: (#none).
	unnamedMorph3 frameFractions: (0@0 corner: 1@1).
	unnamedMorph3 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph3 balloonText: (nil).
	unnamedMorph3 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.945)).
	unnamedMorph3 color: (Color blue).
	unnamedMorph3 borderWidth: (0).
	unnamedMorph3 borderStyle2: (#simple).
	unnamedMorph3 borderColor: (Color black).
	unnamedMorph3 orientation: (#horizontal).
	unnamedMorph2 addChild: unnamedMorph3.
	changeUserButton := UiPushButton new.
	changeUserButton changeTableLayout.
	changeUserButton extent: (50@21).
	changeUserButton position: (50@10).
	changeUserButton hResizing: (#shrinkWrap).
	changeUserButton vResizing: (#shrinkWrap).
	changeUserButton minWidth: (2).
	changeUserButton minHeight: (2).
	changeUserButton cellInset: (4).
	changeUserButton cellPositioning: (#center).
	changeUserButton cellSpacing: (#none).
	changeUserButton layoutInset: (2@0 corner: 2@0).
	changeUserButton listCentering: (#topLeft).
	changeUserButton listDirection: (#leftToRight).
	changeUserButton listSpacing: (#none).
	changeUserButton maxCellSize: (1073741823).
	changeUserButton minCellSize: (0).
	changeUserButton wrapCentering: (#topLeft).
	changeUserButton wrapDirection: (#none).
	changeUserButton frameFractions: (0@0 corner: 1@1).
	changeUserButton frameOffsets: (0@0 corner: 0@0).
	changeUserButton balloonText: (nil).
	changeUserButton balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.941)).
	changeUserButton enabled: (true).
	changeUserButton setProperty: #iconByDesigner toValue: 'nil'.
	changeUserButton icon: (nil).
	changeUserButton text: ('Change').
	changeUserButton color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	changeUserButton checkable: (false).
	changeUserButton checked: (false).
	changeUserButton autoExclusive: (false).
	changeUserButton setProperty: #objectName toValue: 'changeUserButton'.
	unnamedMorph2 addChild: changeUserButton.
	unnamedMorph1 addChild: unnamedMorph2.
	statisticsTable := UiTableView new.
	statisticsTable extent: (100@62).
	statisticsTable position: (6@41).
	statisticsTable hResizing: (#spaceFill).
	statisticsTable vResizing: (#spaceFill).
	statisticsTable minWidth: (2).
	statisticsTable minHeight: (2).
	statisticsTable cellInset: (0).
	statisticsTable cellPositioning: (#center).
	statisticsTable cellSpacing: (#none).
	statisticsTable layoutInset: (0).
	statisticsTable listCentering: (#topLeft).
	statisticsTable listDirection: (#topToBottom).
	statisticsTable listSpacing: (#none).
	statisticsTable maxCellSize: (1073741823).
	statisticsTable minCellSize: (0).
	statisticsTable wrapCentering: (#topLeft).
	statisticsTable wrapDirection: (#none).
	statisticsTable frameFractions: (0@0 corner: 1@1).
	statisticsTable frameOffsets: (0@0 corner: 0@0).
	statisticsTable balloonText: (nil).
	statisticsTable balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.945)).
	statisticsTable widgetResizable: (false).
	statisticsTable horizontalScrollBarPolicy: (#asNeeded).
	statisticsTable verticalScrollBarPolicy: (#asNeeded).
	statisticsTable outerBorderWidth: (0).
	statisticsTable scrollBarThickness: (9).
	statisticsTable modelClass: (nil).
	statisticsTable horizontalHeaderVisible: (true).
	statisticsTable verticalHeaderVisible: (true).
	statisticsTable selectionBehavior: (#selectCells).
	statisticsTable selectionMode: (UiViewSingleSelection).
	statisticsTable setProperty: #objectName toValue: 'statisticsTable'.
	unnamedMorph1 addChild: statisticsTable.
	container addChild: unnamedMorph1.
	workSpace := UiScrollArea new.
	workSpace extent: (213@416).
	workSpace position: (329@4).
	workSpace hResizing: (#spaceFill).
	workSpace vResizing: (#spaceFill).
	workSpace minWidth: (2).
	workSpace minHeight: (2).
	workSpace cellInset: (0).
	workSpace cellPositioning: (#center).
	workSpace cellSpacing: (#none).
	workSpace layoutInset: (0).
	workSpace listCentering: (#topLeft).
	workSpace listDirection: (#topToBottom).
	workSpace listSpacing: (#none).
	workSpace maxCellSize: (1073741823).
	workSpace minCellSize: (0).
	workSpace wrapCentering: (#topLeft).
	workSpace wrapDirection: (#none).
	workSpace frameFractions: (0.6@0 corner: 1@1).
	workSpace frameOffsets: (2@2 corner: -2@ -2).
	workSpace balloonText: (nil).
	workSpace balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	workSpace widgetResizable: (false).
	workSpace horizontalScrollBarPolicy: (#asNeeded).
	workSpace verticalScrollBarPolicy: (#asNeeded).
	workSpace outerBorderWidth: (0).
	workSpace scrollBarThickness: (9).
	workSpace setProperty: #objectName toValue: 'workSpace'.
	container addChild: workSpace.
	unnamedMorph4 := UiContainer new.
	unnamedMorph4 changeTableLayout.
	unnamedMorph4 extent: (217@420).
	unnamedMorph4 position: (110@2).
	unnamedMorph4 hResizing: (#spaceFill).
	unnamedMorph4 vResizing: (#spaceFill).
	unnamedMorph4 minWidth: (2).
	unnamedMorph4 minHeight: (2).
	unnamedMorph4 cellInset: (0).
	unnamedMorph4 cellPositioning: (#center).
	unnamedMorph4 cellSpacing: (#none).
	unnamedMorph4 layoutInset: (4).
	unnamedMorph4 listCentering: (#topLeft).
	unnamedMorph4 listDirection: (#topToBottom).
	unnamedMorph4 listSpacing: (#none).
	unnamedMorph4 maxCellSize: (1073741823).
	unnamedMorph4 minCellSize: (0).
	unnamedMorph4 wrapCentering: (#topLeft).
	unnamedMorph4 wrapDirection: (#none).
	unnamedMorph4 frameFractions: (0.2@0 corner: 0.6@1).
	unnamedMorph4 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph4 balloonText: (nil).
	unnamedMorph4 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	unnamedMorph4 color: (Color transparent).
	unnamedMorph4 borderWidth: (0).
	unnamedMorph4 borderStyle2: (#simple).
	unnamedMorph4 borderColor: (Color black).
	taskDescription := UiScrollArea new.
	taskDescription extent: (209@385).
	taskDescription position: (114@6).
	taskDescription hResizing: (#spaceFill).
	taskDescription vResizing: (#spaceFill).
	taskDescription minWidth: (2).
	taskDescription minHeight: (2).
	taskDescription cellInset: (0).
	taskDescription cellPositioning: (#topLeft).
	taskDescription cellSpacing: (#none).
	taskDescription layoutInset: (0).
	taskDescription listCentering: (#topLeft).
	taskDescription listDirection: (#topToBottom).
	taskDescription listSpacing: (#none).
	taskDescription maxCellSize: (1073741823).
	taskDescription minCellSize: (0).
	taskDescription wrapCentering: (#topLeft).
	taskDescription wrapDirection: (#none).
	taskDescription frameFractions: (0@0 corner: 1@1).
	taskDescription frameOffsets: (0@0 corner: 0@0).
	taskDescription balloonText: (nil).
	taskDescription balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	taskDescription widgetResizable: (false).
	taskDescription horizontalScrollBarPolicy: (#asNeeded).
	taskDescription verticalScrollBarPolicy: (#asNeeded).
	taskDescription outerBorderWidth: (0).
	taskDescription scrollBarThickness: (9).
	taskDescription setProperty: #objectName toValue: 'taskDescription'.
	unnamedMorph4 addChild: taskDescription.
	unnamedMorph5 := UiContainer new.
	unnamedMorph5 changeTableLayout.
	unnamedMorph5 extent: (209@26).
	unnamedMorph5 position: (114@391).
	unnamedMorph5 hResizing: (#spaceFill).
	unnamedMorph5 vResizing: (#shrinkWrap).
	unnamedMorph5 minWidth: (2).
	unnamedMorph5 minHeight: (2).
	unnamedMorph5 cellInset: (2).
	unnamedMorph5 cellPositioning: (#center).
	unnamedMorph5 cellSpacing: (#none).
	unnamedMorph5 layoutInset: (2).
	unnamedMorph5 listCentering: (#topLeft).
	unnamedMorph5 listDirection: (#leftToRight).
	unnamedMorph5 listSpacing: (#none).
	unnamedMorph5 maxCellSize: (1073741823).
	unnamedMorph5 minCellSize: (0).
	unnamedMorph5 wrapCentering: (#topLeft).
	unnamedMorph5 wrapDirection: (#none).
	unnamedMorph5 frameFractions: (0@0 corner: 1@1).
	unnamedMorph5 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph5 balloonText: (nil).
	unnamedMorph5 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	unnamedMorph5 color: (Color transparent).
	unnamedMorph5 borderWidth: (0).
	unnamedMorph5 borderStyle2: (#simple).
	unnamedMorph5 borderColor: (Color black).
	unnamedMorph6 := UiSpacer new.
	unnamedMorph6 extent: (57@20).
	unnamedMorph6 position: (116@394).
	unnamedMorph6 hResizing: (#spaceFill).
	unnamedMorph6 vResizing: (#rigid).
	unnamedMorph6 minWidth: (2).
	unnamedMorph6 minHeight: (2).
	unnamedMorph6 cellInset: (0).
	unnamedMorph6 cellPositioning: (#center).
	unnamedMorph6 cellSpacing: (#none).
	unnamedMorph6 layoutInset: (0).
	unnamedMorph6 listCentering: (#topLeft).
	unnamedMorph6 listDirection: (#topToBottom).
	unnamedMorph6 listSpacing: (#none).
	unnamedMorph6 maxCellSize: (1073741823).
	unnamedMorph6 minCellSize: (0).
	unnamedMorph6 wrapCentering: (#topLeft).
	unnamedMorph6 wrapDirection: (#none).
	unnamedMorph6 frameFractions: (0@0 corner: 1@1).
	unnamedMorph6 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph6 balloonText: (nil).
	unnamedMorph6 balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.929)).
	unnamedMorph6 color: (Color blue).
	unnamedMorph6 borderWidth: (0).
	unnamedMorph6 borderStyle2: (#simple).
	unnamedMorph6 borderColor: (Color black).
	unnamedMorph6 orientation: (#horizontal).
	unnamedMorph5 addChild: unnamedMorph6.
	check := UiPushButton new.
	check changeTableLayout.
	check extent: (70@21).
	check position: (175@393).
	check hResizing: (#rigid).
	check vResizing: (#rigid).
	check minWidth: (2).
	check minHeight: (2).
	check cellInset: (2).
	check cellPositioning: (#center).
	check cellSpacing: (#none).
	check layoutInset: (2).
	check listCentering: (#topLeft).
	check listDirection: (#leftToRight).
	check listSpacing: (#none).
	check maxCellSize: (1073741823).
	check minCellSize: (0).
	check wrapCentering: (#topLeft).
	check wrapDirection: (#none).
	check frameFractions: (0@0 corner: 1@1).
	check frameOffsets: (0@0 corner: 0@0).
	check balloonText: (nil).
	check balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	check enabled: (true).
	check setProperty: #iconByDesigner toValue: 'nil'.
	check icon: (nil).
	check text: ('Check').
	check color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	check checkable: (false).
	check checked: (false).
	check autoExclusive: (false).
	check setProperty: #objectName toValue: 'check'.
	unnamedMorph5 addChild: check.
	reset := UiPushButton new.
	reset changeTableLayout.
	reset extent: (70@21).
	reset position: (249@393).
	reset hResizing: (#rigid).
	reset vResizing: (#rigid).
	reset minWidth: (2).
	reset minHeight: (2).
	reset cellInset: (0).
	reset cellPositioning: (#center).
	reset cellSpacing: (#none).
	reset layoutInset: (2).
	reset listCentering: (#topLeft).
	reset listDirection: (#leftToRight).
	reset listSpacing: (#none).
	reset maxCellSize: (1073741823).
	reset minCellSize: (0).
	reset wrapCentering: (#topLeft).
	reset wrapDirection: (#none).
	reset frameFractions: (0@0 corner: 1@1).
	reset frameOffsets: (0@0 corner: 0@0).
	reset balloonText: (nil).
	reset balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	reset enabled: (true).
	reset setProperty: #iconByDesigner toValue: 'nil'.
	reset icon: (nil).
	reset text: ('Reset').
	reset color: ((Color r: 0.8 g: 0.8 b: 0.8)).
	reset checkable: (false).
	reset checked: (false).
	reset autoExclusive: (false).
	reset setProperty: #objectName toValue: 'reset'.
	unnamedMorph5 addChild: reset.
	unnamedMorph4 addChild: unnamedMorph5.
	container addChild: unnamedMorph4.
	tasks := UiTreeView new.
	tasks extent: (104@311).
	tasks position: (4@109).
	tasks hResizing: (#spaceFill).
	tasks vResizing: (#spaceFill).
	tasks minWidth: (2).
	tasks minHeight: (2).
	tasks cellInset: (0).
	tasks cellPositioning: (#center).
	tasks cellSpacing: (#none).
	tasks layoutInset: (0).
	tasks listCentering: (#topLeft).
	tasks listDirection: (#topToBottom).
	tasks listSpacing: (#none).
	tasks maxCellSize: (1073741823).
	tasks minCellSize: (0).
	tasks wrapCentering: (#topLeft).
	tasks wrapDirection: (#none).
	tasks frameFractions: (0@0.25 corner: 0.2@1).
	tasks frameOffsets: (2@2 corner: -2@ -2).
	tasks balloonText: (nil).
	tasks balloonColor: ((TranslucentColor r: 0.91 g: 0.94 b: 1 alpha: 0.922)).
	tasks widgetResizable: (false).
	tasks horizontalScrollBarPolicy: (#asNeeded).
	tasks verticalScrollBarPolicy: (#asNeeded).
	tasks outerBorderWidth: (0).
	tasks scrollBarThickness: (9).
	tasks modelClass: (nil).
	tasks horizontalHeaderVisible: (true).
	tasks verticalHeaderVisible: (false).
	tasks selectionBehavior: (#selectRows).
	tasks selectionMode: (UiViewSingleSelection).
	tasks setProperty: #objectName toValue: 'tasks'.
	container addChild: tasks.
	panel addChild: container.
	panel setProperty: #uiClassName toValue: 'MMMemoryMentor'.! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.5'!
reset
	"No description given."
	
	^ reset! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.48'!
statisticsTable
	"No description given."
	
	^ statisticsTable! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.492'!
taskDescription
	"No description given."
	
	^ taskDescription! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.506'!
tasks
	"No description given."
	
	^ tasks! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.468'!
userLabel
	"No description given."
	
	^ userLabel! !

!MMMemoryMentorUi methodsFor: 'accessing' stamp: 'RL 7/1/2013 13:06:46.488'!
workSpace
	"No description given."
	
	^ workSpace! !

Morph subclass: #MMArrowArea
	instanceVariableNames: 'arrows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMArrowArea methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:54:34.314'!
addArrow: anArrow
	arrows add: anArrow.
	self connect: anArrow signal: #update toSelector: #changed! !

!MMArrowArea methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 14:20:27.945'!
arrowCount
	^ arrows size! !

!MMArrowArea methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 10:05:04.486'!
drawOn: aCanvas
	super drawOn: aCanvas.
	aCanvas
		translateBy: self position
		"clippingTo: self bounds"
		during: [ :canvas |
			arrows do: [ :anArrow | anArrow drawArrowOn: canvas	]
		]! !

!MMArrowArea methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 13:33:23.95'!
initialize 
	super initialize.
	arrows := OrderedCollection new! !

!MMArrowArea methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 12:00:40.178'!
removeArrow: anArrow
	arrows remove: anArrow.
	self changed! !

Morph subclass: #MMCellGroupMorph
	instanceVariableNames: 'cells type displayMode memoryMorph isShowingBits valueMorphs pointerArrow contextMenu cellGroupContainer lastClickedValueMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

MMCellGroupMorph class
	instanceVariableNames: 'typeToValidDisplayModes symbolsToMenuString'!

MMCellGroupMorph class
	instanceVariableNames: 'typeToValidDisplayModes symbolsToMenuString'!

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 11:00:07.126'!
bitsReadTypeForType: aSymbol
	(self oneByteTypes includes: aSymbol)
		ifTrue: [^ #raw].

	^ (self eightByteTypes includes: aSymbol)
		ifTrue: [#ulong]
		ifFalse: [#uint]! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/3/2013 15:34:39.221'!
byteHeight
	^ TextStyle default defaultFont height + 6! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 10:56:40.498'!
byteLengthOfType: aSymbol
	^ true caseOf: {
		[(self oneByteTypes includes: aSymbol)] -> [1].
		[(self fourByteTypes includes: aSymbol)] -> [4].
		[(self eightByteTypes includes: aSymbol)] -> [8].
	}! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 11:11:09.818'!
byteWidth
	^ (TextStyle default defaultFont widthOf: $0) * (8+4)! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:23:29.159'!
contentMorphWidthForType: aSymbol
	| widthFactor |
	widthFactor := (self oneByteTypes includes: aSymbol) 
		ifTrue: [1] 
		ifFalse: [4].
	
	^ self byteWidth * widthFactor! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 10:43:54.146'!
defaultDisplayMode
	^ #hexadecimal
	! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 11:15:05.714'!
defaultWidth
	^ self byteWidth * 4! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 10:53:49.963'!
eightByteTypes
	^ #(pointer long ulong double)! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:57:35.886'!
for: cellOrCells in: aMemoryMorph
	 ^ self new
		memoryMorph: aMemoryMorph;
		cells: cellOrCells;
		setup! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:29:18.496'!
for: cellOrCells in: aMemoryMorph displayMode: aSymbol
	 ^ (self for: cellOrCells in: aMemoryMorph)
		displayMode: aSymbol;
		yourself ! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 10:59:00.841'!
fourByteTypes
	^ #(int uint float)! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'WS 5/29/2013 11:05:48.856'!
invalidateValidDisplayModes
	typeToValidDisplayModes := nil! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 10:53:34.311'!
oneByteTypes
	^ #(raw char uchar bool)! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:04:30.198'!
setterForType: type
	^ type caseOf: {
		[ #raw ] -> [ #at:putByte: ].
		[ #pointer ] -> [ #at:putULong: ].
		[ #int ] -> [ #at:putInt: ].
		[ #uint ] -> [ #at:putUInt: ].
		[ #long ] -> [ #at:putLong: ].
		[ #ulong ] -> [ #at:putULong: ].
		[ #char ] -> [ #at:putChar: ].
		[ #uchar ] -> [ #at:putUChar: ].
		[ #float ] -> [ #at:putFloat: ].
		[ #double ] -> [ #at:putDouble: ]
	} otherwise: [ self error: 'unsupported type' ].! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:39:18.252'!
signedIntegerTypes
	^ { #int. #long. #char.	}! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'WS 5/29/2013 11:13:02.922'!
symbolsToMenuString
	symbolsToMenuString ifNil: [
		symbolsToMenuString := Dictionary newFrom: {
			#binary -> 'Binary'.
			#decimal -> 'Decimal'.
			#hexadecimal -> 'Hexadecimal'.
			#ascii -> 'ASCII'.
			#raw -> 'Raw'.
			#pointer -> 'Pointer'.
			#int -> 'Integer'.
			#uint -> 'Unsigned Integer'.
			#long -> 'Long'.
			#ulong -> 'Unsigned Long'.
			#char -> 'Character'.
			#uchar -> 'Unsigned Character'.
			#float -> 'Float'.
			#double -> 'Double'
		}
	].
	
	^ symbolsToMenuString! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'WS 6/27/2013 13:07:24.418'!
typeToValidDisplayModes
	^ typeToValidDisplayModes ifNil: 
		[typeToValidDisplayModes := 
			{#raw -> #(hexadecimal binary decimal ascii).
			#pointer -> #(hexadecimal).
			#int -> #(decimal binary hexadecimal).
			#uint -> #(decimal binary hexadecimal).
			#long -> #(decimal binary hexadecimal).
			#ulong -> #(decimal binary hexadecimal).
			#char -> #(binary decimal hexadecimal).
			#uchar -> #(ascii decimal binary hexadecimal).
			#float -> #(decimal).
			#double -> #(decimal)}.
		typeToValidDisplayModes].! !

!MMCellGroupMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:13:28.695'!
widthForBytes: aNumber
	^ self byteWidth * aNumber! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:51:55.534'!
address
	^ cells first address! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'WS 6/25/2013 11:50:14.078'!
addressFor: aValueMorph
	^ cells first address + ((valueMorphs indexOf: aValueMorph) - 1)! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 13:06:45.602'!
annotate
	lastClickedValueMorph ifNotNil: [ lastClickedValueMorph annotate ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:50:43.167'!
byteValueAt: address type: aSymbol
	^ self 
		contentValueAt: address 
		type: aSymbol! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 11:46:38.254'!
cells
	 ^ cells! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:56:33'!
cells: cellOrCells
	cells := cellOrCells isCollection
				ifTrue: [ cellOrCells ]
				ifFalse: [ {cellOrCells} ].
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:06:47.602'!
changeValue
	lastClickedValueMorph ifNotNil: [ | newValue |
		newValue := FillInTheBlankMorph request: 'New Value' initialAnswer: lastClickedValueMorph contentString.
		newValue ifNotEmpty: [ self updateContentOf: lastClickedValueMorph with:  newValue ] ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:48:37.848'!
contentValueAt: address type: aSymbol
	^ aSymbol caseOf: {
		[ #pointer ] -> [ self memory ulongAt: address ].
		[ #long ] -> [ self memory longAt: address ].	
		[ #ulong ] -> [ self memory ulongAt: address ].
		[ #int ] -> [ self memory intAt: address ].
		[ #uint ] -> [ self memory uintAt: address ].
		[ #float ] -> [ self memory floatAt: address ].
		[ #double ] -> [ self memory doubleAt: address ].
		[ #char ] -> [ self memory charAt: address ].
		[ #uchar ] -> [ self memory ucharAt: address ].
		[ #bool ] -> [ self memory byteAt: address ].
		[ #raw ] -> [ self memory byteAt: address ].
	} otherwise: [ 0 ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:50:14.723'!
contentValueForType: aSymbol
	^ self 
		contentValueAt: self address 
		type: aSymbol  ! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 7/2/2013 00:39:25.831'!
copyValueToClipboard
	lastClickedValueMorph ifNotNil: [Clipboard clipboardText: lastClickedValueMorph contentString].
	! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 14:21:06.672'!
createAddressMorph
	| width labelMorph |
	width := (TextStyle default defaultFont widthOf: $0) * 
			(('0x', (cells first memory endAddress printStringBase: 16)) size + 1).
	labelMorph := MMMemoryAddressMorph 
		forAddress: cells first address 
		width: width 
		height: self class byteHeight.
	"self updateTooltipWith: labelMorph addressString."
	^ labelMorph! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:24:13.492'!
createBitsMorphFor: numberValue byteLength: numberOfBytes
	^ MMBitsMorph
		content: numberValue
		extent: (self class contentMorphWidthForType: self type) @ (self class byteHeight)
		byteLength: numberOfBytes! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:57:23.097'!
createByteMorph: i of: aCell type: aSymbol
	| interpretedValue bitsValue |
	interpretedValue := self 
		byteValueAt: aCell address + i 
		type: aSymbol.
	bitsValue := self 
		byteValueAt: aCell address + i 
		type: #raw.
	
	^ self 
		createContentMorphFor: interpretedValue 
		withBits: bitsValue! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 13:53:37.173'!
createByteMorphs
	| byteMorphs |
	byteMorphs := OrderedCollection new.
	cells do: [ :cell | 0 to: 3 do: [ :i | 
			(byteMorphs add: (self createByteMorph: i of: cell type: type)) ]].
	^ byteMorphs! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:57:45.276'!
createCellMorph
	| value readType bitsValue |
	value := self contentValueForType: type.
	readType := self class bitsReadTypeForType: type.
	bitsValue := self contentValueForType: readType.
	
	^ self 
		createContentMorphFor: value 
		withBits: bitsValue! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 11:00:41.784'!
createContent
	^ (self class oneByteTypes includes: type)
			ifTrue: [ self createByteMorphs ]
			ifFalse: [ {self createCellMorph} ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:00:47.008'!
createContentMorphFor: value withBits: bitsValue
	|interpretedValueMorph |
	interpretedValueMorph := self createValueMorphFor: value.
	valueMorphs add: interpretedValueMorph.
	
	^ self isShowingBits 
		ifTrue: [|bitsMorph|
			bitsMorph := self 
				createBitsMorphFor: bitsValue
				byteLength: (self class byteLengthOfType: type).
			self 
				createVerticalCellContainerHolding: bitsMorph 
				and: interpretedValueMorph]
		ifFalse: [interpretedValueMorph]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 7/2/2013 00:28:13.982'!
createContextMenu
	^ MenuMorph new
		add: 'Change Value' target: self selector: #changeValue;
		add: 'Copy Value to Clipboard' target: self selector: #copyValueToClipboard;
		add: 'Annotate' target: self selector: #annotate;
		addLine;
		add: 'Interpret memory as ...' subMenu: self createDisplayModeMenu;
		addUpdating: #isShowingBitsMenuString target: self action: #toggleShowingBits;
		add: 'Retarget pointer' target: self selector: #retargetPointer;
		addLine;
		add: 'Show bits for all' target: memoryMorph selector: #showBitsForAllCellGroups;
		add: 'Hide bits for all' target: memoryMorph selector: #hideBitsForAllCellGroups;
		add: 'Show raw memory as ...' subMenu: self createShowRawBitsMenu;
		yourself! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:34:54.218'!
createDisplayModeMenu
	| menu |
	menu := MenuMorph new.
	self class typeToValidDisplayModes do: [ :each | | subMenu type displayModes |
		type := each key.
		displayModes := each value.		
		subMenu := MenuMorph new.
		displayModes do: [ :displayMode |
			subMenu add: (self class symbolsToMenuString at: displayMode) target: self selector: #type:displayMode: argumentList: {type. displayMode}
		].
		menu add: (self class symbolsToMenuString at: type) subMenu: subMenu
	].
	menu addLine.
	menu add: 'c-string' target: self selector: #showString.
	^ menu! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 14:23:36.195'!
createHorizontalCellGroupContainer
	^ BorderedMorph new
		layoutPolicy: TableLayout new;
		color: (Color gray: 0.9);
		borderWidth: 1;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		listCentering: #left;
		yourself! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:02:14.157'!
createShowRawBitsMenu
	^ MenuMorph new
		add: (self class symbolsToMenuString at: #binary) target: self memoryMorph selector: #displayModeForRawMemory: argument: #binary;
		add: (self class symbolsToMenuString at: #hexadecimal) target: self memoryMorph selector: #displayModeForRawMemory: argument: #hexadecimal;
		yourself! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:10:44.11'!
createValueMorphFor: aNumber
	^ MMValueMorph 
		content: aNumber
		extent: (self class contentMorphWidthForType: self type) @ (self class byteHeight)
		displayMode: self displayMode
		alwaysShowSign: (self class signedIntegerTypes includes: type)! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:59:14.336'!
createVerticalCellContainerHolding: upperMorph and: lowerMorph
	^ Morph new
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		wrapDirection: #topToBottom;
		listCentering: #center;
		addMorphBack: upperMorph;
		addMorphBack: lowerMorph;
		yourself! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:17:29.75'!
delete
	super delete.
	pointerArrow ifNotNil: [:arrow | arrow remove]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 15:18:28.708'!
displayMode
	^ displayMode! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 10:49:45.629'!
displayMode: aSymbol
	(displayMode ~= aSymbol) ifTrue:
		[displayMode := aSymbol.
		self displayModeChanged]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 14:16:28.098'!
displayModeChanged 
	valueMorphs do: [ :valueMorph | valueMorph displayMode: displayMode ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 14:16:21.419'!
hasPointer
	^ pointerArrow notNil! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 11:22:22.398'!
hideBits
	isShowingBits := false.
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:54:37.018'!
initialize 
	super initialize.
	
	valueMorphs := OrderedCollection new.
	type := #raw.
	isShowingBits := false.
	cellGroupContainer := self createHorizontalCellGroupContainer.
	self initializeLayout.
	self
		displayMode: self class defaultDisplayMode;
		color: Color white;
		wantsYellowButtonMenu: true! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:48:38.283'!
initializeLayout
	self
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		listCentering: #left! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/4/2013 16:48:44.953'!
isShowingBits
	^ isShowingBits! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 7/8/2013 10:26:34.582'!
isShowingBitsMenuString
	^ (isShowingBits 
		ifTrue: ['Hide'] 
		ifFalse: ['Show']), ' bits'
! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 10:46:55.269'!
isValidPointer
	^ type == #pointer and: [|address| 
		address := self contentValueForType: #pointer.
		(self memoryMorph groupAtAddress: address) notNil]! !

!MMCellGroupMorph methodsFor: 'accessing' stamp: 'RL 5/22/2013 13:51:17.422'!
memory
	^ memoryMorph memory ! !

!MMCellGroupMorph methodsFor: 'accessing' stamp: 'RL 5/15/2013 14:36:58.93'!
memoryMorph
	^ memoryMorph! !

!MMCellGroupMorph methodsFor: 'accessing' stamp: 'RL 5/15/2013 14:36:58.94'!
memoryMorph: anObject
	memoryMorph := anObject! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 14:18:43.002'!
mergeToSize: size
	memoryMorph merge: self toSize: size! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:00:46.8'!
parseValue: aString
	^ [ displayMode caseOf: {
		[ #binary ] -> [ Integer readFrom: ((aString endsWith: 'b')
			ifTrue: [ aString allButLast ]
			ifFalse: [ aString ]) base: 2 ].
		[ #decimal ] -> [ (type == #float or: [ type == #double ])
			ifTrue: [ Float readFrom: aString ]
			ifFalse: [Integer readFrom: aString base: 10 ] ].
		[ #hexadecimal ] -> [ Integer readFrom: ((aString beginsWith: '0x')
			ifTrue: [ aString allButFirst: 2 ]
			ifFalse: [ aString ]) base: 16 ].
		[ #ascii ] -> [ | strippedString|
			strippedString := ((aString beginsWith: '''') and: [ aString endsWith: '''' ] and: [ aString size > 2])
				ifTrue: [ aString allButLast allButFirst ]
				ifFalse: [ aString ].
			(strippedString beginsWith: '\')
				ifTrue: [ Integer readFrom: strippedString allButFirst base: 10 ]
				ifFalse: [ strippedString first asciiValue ]
		]
	} otherwise: [ nil ] ] on: Error do: [ ^ nil ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 16:09:33.718'!
pointerArrow
	^ pointerArrow! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 14:22:00.578'!
pointerArrow: aPointerArrow
	pointerArrow ifNotNil: [ pointerArrow remove ].
	pointerArrow := aPointerArrow! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:59:46.184'!
removePointerArrow
	pointerArrow ifNotNil: [ pointerArrow remove. pointerArrow := nil ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 14:19:40.323'!
retargetPointer
	self removePointerArrow.
	MMPointerArrow from: self! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 5/23/2013 14:04:18.002'!
retargetPointerTo: anAddress
	| targetGroup |
	targetGroup := memoryMorph groupAtAddress: anAddress.
	targetGroup ifNil: [ ^ self ].
	
	MMPointerArrow from: self to: targetGroup! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 7/2/2013 20:53:13.48'!
retargetedPointer: aMemoryCell
	self memory at: self address putULong: aMemoryCell address.
	self type: #pointer displayMode: #hexadecimal.
	lastClickedValueMorph content: (self contentValueForType: #pointer)! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 13:55:36.55'!
rightCenterInWorld
	^ self pointInWorld: self rightCenter! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 12:54:37.032'!
setup
	self
		addMorphBack: self createAddressMorph;
		addMorphBack: cellGroupContainer;
		updateContent;
		extent: self fullBounds extent.

	contextMenu := self createContextMenu.! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 11:22:14.228'!
showBits
	isShowingBits := true.
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:34:31.35'!
showString
	| nullByteFound currentAddress |
	nullByteFound := false.
	currentAddress := self address.
	[ nullByteFound ] whileFalse: [
		(memoryMorph groupAtAddress: currentAddress) ifNil: [ ^ self ] ifNotNil: [ :group | group type: #uchar displayMode: #ascii ].
		0 to: 3 do: [ :i | 
			nullByteFound := nullByteFound or: [ (memoryMorph memory byteAt: currentAddress + i) = 0 ]
		].
		currentAddress := currentAddress + 4
	]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:44:44.176'!
split
	memoryMorph split: self! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/4/2013 17:12:16.565'!
toggleShowingBits
	isShowingBits := isShowingBits not.
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/3/2013 16:40:56.11'!
type
	^ type! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 10:49:15.521'!
type: aSymbol
	(type ~= aSymbol) ifTrue: 
		[type := aSymbol.
		self typeChanged] ! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'WS 5/29/2013 11:19:02.336'!
type: typeSymbol displayMode: displayModeSymbol
	self type: typeSymbol.
	self displayMode: displayModeSymbol! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 14:18:55.473'!
typeChanged
	(self class eightByteTypes includes: type)
		ifTrue: [ self mergeToSize: 2 ]
		ifFalse: [ self split ].
		
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 10:50:22.336'!
updateContent
	self removePointerArrow.
	
	cellGroupContainer submorphsDo: #delete.
	valueMorphs := OrderedCollection new.
	cellGroupContainer minHeight: self class byteHeight * cells size.
	cellGroupContainer addMorphs: self createContent.
	
	(self isValidPointer and: [self hasPointer not]) 
		ifTrue: [self retargetPointerTo: (self contentValueForType: #pointer)].
	
	self memoryMorph updateExtents
! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:04:12.93'!
updateContentOf: aValueMorph with: aString
	| newValue selector |
	
	newValue := self parseValue: aString.
	newValue ifNil: [ ^ self ].
	
	selector := self class setterForType: type.
	
	self memory perform: selector with: (self addressFor: aValueMorph) with: newValue.
	self updateContent! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'WS 6/29/2013 21:38:50.642'!
valueMorphAtAddress: anAddress
	^ valueMorphs at: (anAddress - self address) + 1 ifAbsent: [ nil ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'lt 7/2/2013 00:38:25.987'!
valueMorphUnderHand
	| deepestMorphUnderHand morphsUnderHand |
	morphsUnderHand := self morphsAt: (self globalPointToLocal: self currentHand position).
	deepestMorphUnderHand := morphsUnderHand firstOrNil.
	^ deepestMorphUnderHand ifNotNil: [ (deepestMorphUnderHand isKindOf: MMValueMorph)
			ifTrue: [ deepestMorphUnderHand ]
			ifFalse: [ deepestMorphUnderHand submorphs
				detect: [ :morph | morph isKindOf: MMValueMorph ]
				ifNone: [ deepestMorphUnderHand owner submorphs 
					detect: [ :morph | morph isKindOf: MMValueMorph ]
					ifNone: [ nil ]] ] ]! !

!MMCellGroupMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 11:46:07.248'!
yellowButtonActivity: shiftKeyState
	lastClickedValueMorph := self valueMorphUnderHand.
	contextMenu popUpInWorld.! !

Morph subclass: #MMMemoryAddressMorph
	instanceVariableNames: 'address stringMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMMemoryAddressMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 11:57:11.479'!
defaultFont 
	^ StrikeFont familyName: 'Bitmap Envy Code R' pointSize: 10 emphasized: 0! !

!MMMemoryAddressMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 11:59:09.436'!
forAddress: anInteger width: width height: height
	^ self new
		setAddress: anInteger
		width: width 
		height: height! !

!MMMemoryAddressMorph methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 10:58:21.63'!
address
	^ address! !

!MMMemoryAddressMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:08:16.73'!
address: anInteger
	address := anInteger.
	self updateStringMorph! !

!MMMemoryAddressMorph methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 11:02:33.592'!
addressString
	 ^ '0x', (address printStringBase: 16)! !

!MMMemoryAddressMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:09:01.268'!
setAddress: anInteger width: width height: height 
	self
		address: anInteger;
		color: Color white;
		extent: width@height;
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		listCentering: #center.

	stringMorph := StringMorph 
		contents: self addressString 
		font: self class defaultFont.
	self addMorph: stringMorph! !

!MMMemoryAddressMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:08:51.894'!
updateStringMorph
	stringMorph ifNil: [ ^ self ].
	stringMorph contents: self addressString! !

Morph subclass: #MMMemoryMorph
	instanceVariableNames: 'memory groupMorphs cellArea pointerArea'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMMemoryMorph commentStamp: 'RL 5/15/2013 14:38:05.526' prior: 0!
(MMMemoryMorph for: (MMMemory withStartAddress: 16r100 bytes:  #[104 101 108 108 111] cellCount: 20)) openInHand!

!MMMemoryMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 13:14:59.09'!
for: memory
	^ self new
		setupMemory: memory;
		yourself! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 14:20:34.336'!
arrowCount
	^ pointerArea arrowCount! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 11:39:46.13'!
createCellArea	
	cellArea := Morph new.
	cellArea
		color: Color red;
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		wrapDirection: #topToBottom;
		listCentering: #center;
		setProperty: #wantsYellowButtonMenu toValue: false.
	self addMorphBack: cellArea.! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 13:34:24.687'!
createPointerArea	
	pointerArea := MMArrowArea new.
	pointerArea 
		color: Color white;
		width: 160.
	self addMorphBack: pointerArea.! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/3/2013 16:42:48.113'!
displayModeForRawMemory: aSymbol
	groupMorphs 
		select: [:each | each type == #raw]
		thenDo: [:each | each displayMode: aSymbol]! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:08:36.956'!
groupAtAddress: anAddress
	^ groupMorphs 
		detect: [ :each | each address = anAddress ]
		ifNone: [ nil ]! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 11:21:01.747'!
hideBitsForAllCellGroups
	groupMorphs do: #hideBits! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 13:31:42.41'!
initialize
	super initialize.
	
	groupMorphs := OrderedCollection new.
	
	self
		color: Color red;
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom;
		wrapDirection: #leftToRight;
		listCentering: #center.
	
	self
		createCellArea;
		createPointerArea
	! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:52:34.346'!
memory
	^ memory! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/3/2013 13:02:34.86'!
memory: aMemory
	memory := aMemory! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:30:13.696'!
merge: aGroup toSize: anInteger
	| index groupsToBeMerged |
	self splitEnoughGroupsAfterGroup: aGroup forSize: anInteger.
	index := groupMorphs indexOf: aGroup.
	groupsToBeMerged := groupMorphs copyFrom: index to: index + anInteger - 1.
	self mergeCellGroups: groupsToBeMerged! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:20:09.502'!
mergeCellGroups: cellGroups
	| cells |
	cells := cellGroups flatCollect: #cells.
	groupMorphs removeAll: cellGroups allButFirst.
	cellGroups allButFirst do: #delete.
	cellGroups first cells: cells! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 11:36:40.402'!
pointerArea
	^ pointerArea! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:13:12.472'!
pointerArrows
	^ groupMorphs select: [ :cellGroup | cellGroup pointerArrow notNil ] thenCollect: [ :cellGroup | cellGroup pointerArrow ]! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/26/2013 13:32:56.132'!
setupMemory: aMemory
	memory := aMemory.
	memory cellsDo: [ :cell | | groupMorph |
		groupMorph := MMCellGroupMorph for: cell in: self.
		groupMorphs add: groupMorph.
	].

	cellArea addMorphs: groupMorphs.
	self updateExtents! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 11:21:13.605'!
showBitsForAllCellGroups
	groupMorphs do: #showBits! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 11:12:16.377'!
split: aGroup
	| index newGroups |
	aGroup cells size < 2 ifTrue: [^ self].
	
	index := groupMorphs indexOf: aGroup.
	newGroups := aGroup cells allButFirst collect: [ :cell | 
		MMCellGroupMorph for: cell in: self displayMode: aGroup displayMode].
	newGroups reverseDo: [ :group |
		groupMorphs add: group afterIndex: index.
		cellArea addMorph: group after: aGroup].
	aGroup cells: {aGroup cells first}! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:14:34.541'!
splitEnoughGroupsAfterGroup: aGroup forSize: anInteger
	| index toSplit count i  |
	index := index := groupMorphs indexOf: aGroup.
	toSplit := OrderedCollection new.
	count := 0.
	i := 0.
	[ count < anInteger ] whileTrue: [ | group |
		group := groupMorphs at: index + i ifAbsent: [ self error: 'caution!! you are about to leave the memory.' ].
		toSplit add: group.
		count := count + group cells size.
		i := i + 1.
	].
	toSplit do: [ :each | self split: each]! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:06:42.244'!
state
	^ MMMemoryState for: self! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 17:13:33.82'!
updateExtents
	self bounds: self fullBounds.
	pointerArea height: cellArea height.
	
	self pointerArrows do: [ :arrow | arrow updateArrow ]! !

!MMMemoryMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:30:32.783'!
wantsYellowButtonMenu
	^ false! !

Morph subclass: #MMRightAngledArrow
	instanceVariableNames: 'from to spacing arrowBorderWidth points horizontalLungeDirection verticalLungeDirection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMRightAngledArrow class methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:53:48.352'!
from: aPoint to: anotherPoint
	^ self from: aPoint to: anotherPoint spacing: 15! !

!MMRightAngledArrow class methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 14:01:37.892'!
from: aPoint to: anotherPoint spacing: anInteger
	^ super new
		spacing: anInteger;
		initializeFrom: aPoint to: anotherPoint;
		yourself! !

!MMRightAngledArrow class methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 13:58:08.482'!
new
	^ self shouldNotImplement! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:55:44.908'!
arrowBoundsAt: endPoint from: priorPoint 
	"Answer a triangle oriented along the line from priorPoint to endPoint."
	| d v angle wingBase arrowSpec length width |
	endPoint == priorPoint ifTrue: [ ^ {} ].
	
	v := endPoint - priorPoint.
	angle := v degrees.
	d := arrowBorderWidth max: 1.
	arrowSpec := 4@3.
	length := arrowSpec x abs.  width := arrowSpec y abs.
	wingBase := endPoint + (Point r: d * length degrees: angle + 180.0).
	arrowSpec x >= 0
		ifTrue: [^ {	endPoint.
					wingBase + (Point r: d * width degrees: angle + 125.0).
					wingBase + (Point r: d * width degrees: angle - 125.0) }]
		ifFalse: ["Negative length means concave base."
				^ {	endPoint.
					wingBase + (Point r: d * width degrees: angle + 125.0).
					wingBase.
					wingBase + (Point r: d * width degrees: angle - 125.0) }]! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 11:47:34.906'!
bottomLunge
	^ 0 @ -1! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:49:40.186'!
drawArrowOn: canvas
	points size < 2 ifTrue: [ ^ self ].
	
	points overlappingPairsDo: [ :p1 :p2 |
		canvas line: p1 to: p2 width: arrowBorderWidth color: Color black
	].
	canvas
		drawPolygon: (self arrowBoundsAt: points last from: (points at: points size - 1))
		color: Color white borderWidth: arrowBorderWidth borderColor: Color black! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/25/2013 12:00:38.278'!
drawOn: aCanvas
	self drawArrowOn: aCanvas! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:52:43.828'!
initialize
	super initialize.
	
	arrowBorderWidth := 2.
	spacing := 15.
	points := OrderedCollection new.
	horizontalLungeDirection := #right.
	verticalLungeDirection := #top! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:54:03.038'!
initializeFrom: aPoint to: anotherPoint
	| bounds |
	
	from := aPoint.
	to := anotherPoint.
	
	from ~= to ifTrue:[
		self initializePoints.
		bounds := from rect: to.
		points do: [ :point | bounds := bounds encompass: point ].
		(self arrowBoundsAt: points last from: (points at: points size - 1)) do: [ :point | bounds := bounds encompass: point ].
		self bounds: (bounds outsetBy: arrowBorderWidth).
	] ifFalse: [
		points := OrderedCollection new.
		self bounds: (from rect: to)
	].
	
	self update! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 14:09:49.454'!
initializePoints
	from ifNil: [ ^ self ].
	to ifNil: [ ^ self ].
	
	points := OrderedCollection new.
	
	points add: from.
	
	true caseOf: {
		[ (from x - to x) abs >= spacing and: [ (from y - to y) abs >= spacing ] ] -> [
			(from x - to x) abs < (from y - to y) abs ifTrue: [
				points add: from x @ (from interpolateTo: to at: 0.5) y.
				points add: to x @ (from interpolateTo: to at: 0.5) y.
			] ifFalse: [
				points add: (from interpolateTo: to at: 0.5) x @ from y.
				points add: (from interpolateTo: to at: 0.5) x @ to y.
			]
		].
		[ (from x - to x) abs >= spacing and: [ (from y - to y) abs < spacing ] ] -> [ | tempY |
			tempY := verticalLungeDirection == #top ifTrue: [ (from y max: to y) + spacing ] ifFalse: [ (from y min: to y) - spacing ].
			points add: from x @ tempY.
			points add: to x @ tempY
		].
		[ (from x - to x) abs < spacing and: [ (from y - to y) abs >= spacing ] ] -> [ | tempX |
			tempX := horizontalLungeDirection == #left ifTrue: [ (from x min: to x) - spacing ] ifFalse: [ (from x max: to x) + spacing ].
			points add: tempX @ from y.
			points add: tempX @ to y
		].
		[ (from x - to x) abs < spacing and: [ (from y - to y) abs < spacing ] ] -> [
			"straight line, so add no points"
		]
	}.
	
	points add: to! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 13:58:35.68'!
points
	^ points! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'WS 6/26/2013 11:53:58.502'!
spacing: anInteger
	spacing := anInteger.
	
	self initializePoints! !

!MMRightAngledArrow methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:53:45.736'!
update
	self emit! !

Morph subclass: #MMToolTippedMorph
	instanceVariableNames: 'tooltipMorph tooltip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:48:43.314'!
createTooltipMorph
	tooltipMorph := BorderedMorph new.
	tooltipMorph
		color: Color yellow muchLighter paler;
		borderWidth: 1.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'lt 5/13/2013 13:03:13.85'!
handlesMouseMove: event
	^ true! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:48:36.378'!
hideTooltip
	tooltipMorph delete.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:48:48.91'!
initialize
	super initialize.
	self 
		createTooltipMorph;
		initializeEvents.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'lt 5/13/2013 10:51:29.41'!
initializeEvents
	self on: #mouseEnter send: #mouseEnter to: self.
	self on: #mouseMove send: #moveTooltip to: self.
	self on: #mouseLeave send: #mouseLeave to: self.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'lt 5/13/2013 10:51:19.744'!
mouseEnter
	self addAlarm: #showTooltip after: 1000.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'lt 5/13/2013 10:51:09.543'!
mouseLeave
	self removeAlarm: #showTooltip.
	self hideTooltip.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:48:36.364'!
moveTooltip
	tooltipMorph position: ActiveHand center.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:48:36.37'!
showTooltip
	tooltipMorph position: ActiveHand center.
	World addMorph: tooltipMorph.
	tooltipMorph fadeIn.! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:50:32.808'!
tooltip
	^ tooltip! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:51:11.244'!
tooltip: aString
	tooltip := aString.
	self updateTooltip! !

!MMToolTippedMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/14/2013 12:50:52.348'!
updateTooltip
	tooltipMorph
		removeAllMorphs;
		addMorph: (StringMorph new contents: tooltip; yourself);
		bounds: tooltipMorph fullBounds.! !

!SequenceableCollection methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/14/2013 11:39:43.554'!
firstOrNil
	^ self ifNotEmpty: [ self first ] ifEmpty: [ nil ]! !

UiPropertyItemNode subclass: #MMGameMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMGameMode class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:16:01.536'!
freePlay
	^ self
		named: 'Exploration Mode'
		tasks: {
			MMTaskNode zeroMemory.
			MMTaskNode randomMemory
		}! !

!MMGameMode class methodsFor: 'as yet unclassified' stamp: 'lt 7/1/2013 16:50:00.704'!
lessons
	^ self
		named: 'Lessons Mode'
		topics: {
			MMTopic pointers.
			MMTopic structs.
			MMTopic endianness.
			"MMTopic memoryAllocation."
		}! !

!MMGameMode class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:13:13.74'!
named: aName tasks: tasks
	^ self new
		name: aName;
		addDefaultGroup;
		addTasks: tasks;
		yourself! !

!MMGameMode class methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:08:50.586'!
named: aName topics: topics
	^ self new
		name: aName;
		addDefaultGroup;
		addTopics: topics;
		yourself! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:11:38.37'!
addTask: aTask
	self addChild: aTask! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:11:27.72'!
addTasks: aTaskCollection
	aTaskCollection do: [ :topic | self addTask: topic ]! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:01:29.912'!
addTopic: aTopic
	self addChild: aTopic! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:06:34.734'!
addTopics: aTopicCollection
	aTopicCollection do: [ :topic | self addTopic: topic ]! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:01:43.398'!
isGameMode
	^ true! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 12:59:32.276'!
isTask
	^ false! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:01:37.188'!
isTopic
	^ false! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 12:59:32.288'!
name
	^ self at: #name! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 12:59:32.294'!
name: aName
	self at: #name put: aName! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:11:49.856'!
tasks
	^ self childrenInDefaultGroup select: #isTask! !

!MMGameMode methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:02:23.31'!
topics
	^ self childrenInDefaultGroup select: #isTopic! !

UiPropertyItemNode subclass: #MMStatistic
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMStatistic class methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:38:19.738'!
named: aString
	^ self new
		name: aString;
		yourself! !

!MMStatistic class methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:21:25.818'!
new
	^ super new initialize ! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:58:29.018'!
initialize 
	super initialize.
	self value: 0.! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:20:32.314'!
name
	^ self at: #name! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:20:24.318'!
name: aName
	self setProperty: #name to: aName! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:20:09.588'!
setProperty: aSymbol to: anObject
	self at: aSymbol put: anObject.
	self propertiesChanged! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:51:43.438'!
value
	^ self at: #value! !

!MMStatistic methodsFor: 'as yet unclassified' stamp: 'RL 7/1/2013 13:49:48.972'!
value: anObject
	^ self setProperty: #value to: anObject! !

UiPropertyItemNode subclass: #MMTaskNode
	instanceVariableNames: 'selector taskState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMTaskNode class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.254'!
colorForTaskState: aSymbol
	^ (aSymbol caseOf: {
		[#passed] -> [Color green].
		[#failed] -> [Color red].
		[#unattempted] -> [Color white].
	})! !

!MMTaskNode class methodsFor: 'pointers' stamp: 'WS 7/8/2013 13:53:06.872'!
drawPointerArrow
	^ self
		named: 'Simple Pointer'
		selector: #drawPointerArrow! !

!MMTaskNode class methodsFor: 'pointers' stamp: 'WS 7/8/2013 13:53:15.504'!
integerEndianness
	^ self
		named: 'Integer Endianness'
		selector: #integerEndianness! !

!MMTaskNode class methodsFor: 'objects' stamp: 'WS 7/8/2013 13:53:28.512'!
manualInstanceInitialization
	^ self
		named: 'Instance Initialization'
		selector: #manualInstanceInitialization! !

!MMTaskNode class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:52:43.86'!
named: aName selector: aSelector
	^ self new
		name: aName;
		selector: aSelector;
		yourself! !

!MMTaskNode class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.284'!
new
	^ super new initialize ! !

!MMTaskNode class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:10:17.976'!
randomMemory
	^ self
		named: 'Random Memory'
		selector: #randomMemory! !

!MMTaskNode class methodsFor: 'objects' stamp: 'WS 7/8/2013 13:53:59.47'!
structFields
	^ self
		named: 'Struct Fields'
		selector: #structFields! !

!MMTaskNode class methodsFor: 'objects' stamp: 'WS 7/8/2013 13:54:13.488'!
valueOfFieldInStruct
	^ self
		named: 'Value of Field in Struct'
		selector: #valueOfFieldInStruct! !

!MMTaskNode class methodsFor: 'objects' stamp: 'WS 7/8/2013 14:10:08.362'!
zeroMemory
	^ self
		named: 'Zero Memory'
		selector: #zeroMemory! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.112'!
color
	^ self at: #color! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.12'!
color: aColor
	self setProperty: #color to: aColor! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:55:36.094'!
createTask
	^ MMTask perform: selector! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:52:11.748'!
initialize
	taskState := #unattempted.
	self color: Color white! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.138'!
isGameMode
	^ false! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.144'!
isTask
	^ true! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.152'!
isTopic
	^ false! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.156'!
name
	^ self at: #name! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.166'!
name: aName
	self setProperty: #name to: aName! !

!MMTaskNode methodsFor: 'accessing' stamp: 'WS 7/8/2013 14:17:38.75'!
selector
	^ selector! !

!MMTaskNode methodsFor: 'accessing' stamp: 'WS 7/8/2013 14:17:32.482'!
selector: aSymbol
	selector := aSymbol! !

!MMTaskNode methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 13:42:06.176'!
setProperty: aSymbol to: anObject
	self at: aSymbol put: anObject.
	self propertiesChanged! !

!MMTaskNode methodsFor: 'accessing' stamp: 'WS 7/8/2013 13:42:06.238'!
taskState
	"#passed, #failed, or #unattempted"
	^ taskState! !

!MMTaskNode methodsFor: 'accessing' stamp: 'WS 7/8/2013 13:42:06.246'!
taskState: anObject
	taskState := anObject.
	
	self color: (self class colorForTaskState: anObject)! !

UiPropertyItemNode subclass: #MMTopic
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Core'!

!MMTopic class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:16:13.57'!
endianness
	^ self named: 'Endianness' tasks: {
		MMTaskNode integerEndianness
	}
	! !

!MMTopic class methodsFor: 'as yet unclassified' stamp: 'lt 7/1/2013 16:48:40.83'!
memoryAllocation
	^ self named: 'Memory Allocation' tasks: {
		
	}
	! !

!MMTopic class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:48:08.202'!
named: aName tasks: tasks
	^ self new
		name: aName;
		addDefaultGroup;
		addTasks: tasks;
		yourself! !

!MMTopic class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:16:20.436'!
pointers
	^ self named: 'Pointers' tasks: {
		MMTaskNode drawPointerArrow.
	}! !

!MMTopic class methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 14:16:25.14'!
structs
	^ self named: 'Structs' tasks: {
		MMTaskNode structFields.
		MMTaskNode valueOfFieldInStruct.
	}
	! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 14:28:13.242'!
addTask: aTask
	self addChild: aTask! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 14:28:38.142'!
addTasks: aTaskCollection
	aTaskCollection do: [ :task | self addTask: task ]! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 13:01:57.774'!
isGameMode
	^ false! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 15:05:34.136'!
isTask
	^ false! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 15:05:38.716'!
isTopic
	^ true! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 14:22:09.484'!
name
	^ self at: #name! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'RL 6/5/2013 14:22:18.142'!
name: aName
	self at: #name put: aName! !

!MMTopic methodsFor: 'as yet unclassified' stamp: 'lt 6/27/2013 11:46:14.132'!
tasks
	^ self childrenInDefaultGroup! !

UiTextMorph subclass: #MMContentMorph
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

MMContentMorph subclass: #MMBitsMorph
	instanceVariableNames: 'byteLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMBitsMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:32:59.748'!
content: numberValue extent: aRectangle byteLength: numberOfBytes
	^ self new
		extent: aRectangle;
		byteLength: numberOfBytes;
		content: numberValue;
		yourself! !

!MMBitsMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 12:04:11.673'!
defaultFont

	^ StrikeFont familyName: 'Bitmap Envy Code R' pointSize: 9 emphasized: 2! !

!MMBitsMorph methodsFor: 'accessing' stamp: 'lt 6/5/2013 10:36:17.588'!
byteLength
	^ byteLength! !

!MMBitsMorph methodsFor: 'accessing' stamp: 'lt 6/5/2013 10:36:17.597'!
byteLength: anObject
	byteLength := anObject! !

!MMBitsMorph methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 15:57:48.724'!
contentString
	^ self separateBytesOf:
		((1 to: byteLength)
			inject: ''
			into: [ :string :byte | string, (((content digitAt: byte) printStringBase: 2) padded: #left to: 8 with: $0) ])! !

!MMBitsMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 11:56:31.358'!
initialize 
	super initialize.
	self 
		font: self class defaultFont;
		centered! !

!MMBitsMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/14/2013 11:00:49.66'!
separateBytesOf: aString
	| currentIndex |
	currentIndex := 0.
	^ aString inject: '' into: 
		[ :result :each | |separator|
		separator := ''.
		(currentIndex ~= 0) ifTrue: 
				[(currentIndex isDivisibleBy: 8) ifTrue: [separator := ' '].
				(currentIndex isDivisibleBy: 32) ifTrue: [separator := Character cr asString]].
		currentIndex := currentIndex + 1. 
		result , separator , each ]! !

!MMContentMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:53:34.663'!
defaultBackgroundColor
	^ Color gray: 0.9! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:02:02.016'!
content
	^ content! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:02:41.28'!
content: anInteger
	content := anInteger.
	self updateContentString ! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:42:15.314'!
contentString
	^ self subclassResponsibility! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:53:48.108'!
initialize
	super initialize.

	self 
		backgroundColor: self class defaultBackgroundColor;
		layoutPolicy: TableLayout new;
		listDirection: #lefttoRight;
		listCentering: #center;
		editView: Morph new;
		readOnly: true! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:03:03.564'!
updateContentString
	self contentsWrapped: self contentString! !

!MMContentMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/14/2013 16:27:11.807'!
yellowButtonActivity: shiftKeyState
	"to circumvent UiTextMorph's yellowButtonActivity:"
	self owner yellowButtonActivity: shiftKeyState.! !

MMContentMorph subclass: #MMValueMorph
	instanceVariableNames: 'displayMode annotation alwaysShowSign'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Visualization'!

!MMValueMorph class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:57:09.134'!
content: aNumber extent: aRectangle displayMode: aSymbol
	^ self new
		extent: aRectangle;
		displayMode: aSymbol;
		content: aNumber;
		yourself! !

!MMValueMorph class methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:10:22.942'!
content: aNumber extent: aRectangle displayMode: aSymbol alwaysShowSign: aBoolean
	^ self new
		extent: aRectangle;
		displayMode: aSymbol;
		content: aNumber;
		alwaysShowSign: aBoolean;
		yourself! !

!MMValueMorph class methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:33:38.039'!
defaultFont
	^ StrikeFont familyName: 'Bitmap Envy Code R' pointSize: 10 emphasized: 0! !

!MMValueMorph class methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 13:30:30.952'!
displayModes
	^ #(ascii binary decimal hexadecimal)! !

!MMValueMorph methodsFor: 'accessing' stamp: 'rl 7/8/2013 12:09:27.656'!
alwaysShowSign
	^ alwaysShowSign! !

!MMValueMorph methodsFor: 'accessing' stamp: 'rl 7/8/2013 12:09:59.074'!
alwaysShowSign: aBoolean
	alwaysShowSign := aBoolean.
	self updateContentString ! !

!MMValueMorph methodsFor: 'accessing' stamp: 'RL 6/14/2013 13:05:58.24'!
annotate
	self requestAnnotation ifNotNil: [ :response | self annotation: response ]! !

!MMValueMorph methodsFor: 'accessing' stamp: 'RL 6/14/2013 12:40:19.38'!
annotation
	^ annotation! !

!MMValueMorph methodsFor: 'accessing' stamp: 'RL 6/14/2013 12:55:22.422'!
annotation: anObject
	annotation := anObject.
	self balloonText: anObject ! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:19:46.974'!
asciiString
	^ content >= 32
		ifTrue: [ (Character value: content) asString printString ]
		ifFalse: [ (content isZero
			ifTrue: [ '\0' ]
			ifFalse: [ '\x', (content printStringBase: 16) ]) printString ]! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:15:18.252'!
binaryString
	^ content positive
		ifTrue: [ (alwaysShowSign ifTrue: ['+'] ifFalse: ['']), ((content printStringBase: 2) padded: #left to: 8 with: $0), 'b' ]
		ifFalse:[ '-', ((content abs printStringBase: 2) padded: #left to: 8 with: $0), 'b' ]! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 10:17:46.896'!
contentString
	^ (displayMode caseOf: {
		[ #decimal ] -> [ self decimalString ].
		[ #binary ] -> [ self binaryString ].
		[ #hexadecimal ] -> [ self hexString ].
		[ #ascii ] -> [ self asciiString ].
	})! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:12:47.108'!
decimalString
	^ alwaysShowSign
		ifTrue: [ (content >= 0 ifTrue: [ '+' ] ifFalse: [ '' ]), content printString ]
		ifFalse: [ content printString ]! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'RL 5/15/2013 13:34:39.726'!
displayMode
	 ^ displayMode! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 14:02:41.292'!
displayMode: aSymbol
	(self class displayModes includes: aSymbol)
		ifFalse: [ self error: 'invalid display mode' ].
	displayMode := aSymbol.
	self updateContentString ! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:14:38.312'!
hexString
	^ (content positive ifTrue: [ alwaysShowSign ifTrue: ['+'] ifFalse: [''] ] ifFalse: ['-']) , '0x', (content abs printStringBase: 16) padded: #left to: 2 with: $0! !

!MMValueMorph methodsFor: 'as yet unclassified' stamp: 'rl 7/8/2013 12:09:11.422'!
initialize 
	super initialize.

	alwaysShowSign := false.

	self 
		displayMode: #decimal;
		font: self class defaultFont;
		content: 0;
		annotation: '';
		centered.

	self updateContentString ! !

!MMValueMorph methodsFor: 'accessing' stamp: 'lt 6/26/2013 16:50:52.226'!
requestAnnotation
	^ FillInTheBlankMorph
		request: 'Annotation:'
		initialAnswer: annotation
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!UiTextMorph methodsFor: '*MemoryMentor-Visualization' stamp: 'lt 6/27/2013 09:40:57.636'!
mouseUp: evt
	editor ifNotNil: [super mouseUp: evt]! !

TestCase subclass: #MMByteArrayExtensionTest
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 17:37:54.472'!
compare: byteArray atPosition: anInteger
	^ (bytes copyFrom: anInteger to: anInteger + byteArray size - 1) = byteArray! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 11:38:34.118'!
setUp
	bytes := #[110 166 99 74 251 33 9 64 190 0 254 240 66 255 1 255 66 69 1 53 65 69 1 245 190 186 254 202 190 186 254 202 80 250 72 56 0 0 188 65 1 42 0 0 0 0 0 0]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 11:36:50.346'!
testAnotherInt
	bytes := #[0 0 0 0].
	self assert: [ (bytes intAt: 0+1) = 0 ].! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 10:59:53.816'!
testAnotherIntStore
	| byteArray |
	byteArray := #[1 2 3 4].
	byteArray intAt: 1 put: 0.
	self assert: [ byteArray allSatisfy: [:each | each = 0 ]]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 10:57:47.859'!
testAnotherLong
	bytes := #[0 0 0 0 0 0 0 0].
	self assert: [ (bytes longAt: 0+1) = 0 ].! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'lt 6/25/2013 11:00:55.368'!
testAnotherLongStore
	| byteArray |
	byteArray := #[1 2 3 4 5 6 7 8].
	byteArray longAt: 1 put: 0.
	self assert: [ byteArray allSatisfy: [:each | each = 0 ]]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:55:02.67'!
testBool
	self assert: [ (bytes boolAt: 40+1) ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/29/2013 11:14:41.122'!
testBoolStore
	| byteArray |
	byteArray := ByteArray new: 1.
	byteArray boolAt: 1 put: true.
	self assert: [ self compare: byteArray atPosition: 40+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:31:42.152'!
testChar
	self assert: [ (bytes charAt: 32+1) = $P asInteger ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 7/1/2013 11:55:43.278'!
testCharStore
	| byteArray |
	byteArray := ByteArray new: 1.
	byteArray charAt: 1 put: $P asciiValue.
	self assert: [ self compare: byteArray atPosition: 32+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:24:43.294'!
testDouble
	self assert: [ ((bytes doubleAt: 0+1) - 3.14159258) abs < 0.00001 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 6/4/2013 16:50:12.936'!
testDoubleStore
	| byteArray |
	byteArray := ByteArray new: 8.
	byteArray doubleAt: 1 put: 3.14159258.
	self assert: [ self compare: byteArray atPosition: 0+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 14:14:10.732'!
testFloat
	self assert: [ (bytes floatAt: 36+1) = 23.5 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/29/2013 11:18:32.588'!
testFloatStore
	| byteArray |
	byteArray := ByteArray new: 4.
	byteArray floatAt: 1 put: 23.5.
	self assert: [ self compare: byteArray atPosition: 36+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:24:23.3'!
testInt
	self assert: [ (bytes intAt: 8+1) = -251789122 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/29/2013 11:19:08.378'!
testIntStore
	| byteArray |
	byteArray := ByteArray new: 4.
	byteArray intAt: 1 put: -251789122.
	self assert: [ self compare: byteArray atPosition: 8+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:24:52.984'!
testLong
	self assert: [ (bytes longAt: 16+1) = -792275913076030142 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/29/2013 11:19:40.604'!
testLongStore
	| byteArray |
	byteArray := ByteArray new: 8.
	byteArray longAt: 1 put: -792275913076030142.
	self assert: [ self compare: byteArray atPosition: 16+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 13:32:12.53'!
testUChar
	self assert: [ (bytes ucharAt: 33+1) = 250 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/29/2013 11:31:20.054'!
testUCharStore
	| byteArray |
	byteArray := ByteArray new: 1.
	byteArray ucharAt: 1 put: 250.
	self assert: [ self compare: byteArray atPosition: 33+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 12:13:13.44'!
testUint
	self assert: [ (bytes uintAt: 12+1) = 4278320962 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 17:34:55.044'!
testUintStore
	| byteArray |
	byteArray := ByteArray new: 4.
	byteArray uintAt: 1 put: 4278320962.
	self assert: [ self compare: byteArray atPosition: 12+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 12:13:51.114'!
testUlong
	self assert: [ (bytes ulongAt: 24+1) = 14627333968358193854 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 17:38:49.828'!
testUlongStore
	| byteArray |
	byteArray := ByteArray new: 8.
	byteArray ulongAt: 1 put: 14627333968358193854.
	self assert: [ self compare: byteArray atPosition: 24+1 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:42:26.172'!
testYetAnotherInt
	self assert: [ (#[1 0 255 255] intAt: 0+1) = -65535 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:45:53.71'!
testYetAnotherIntStore
	| byteArray |
	byteArray := ByteArray new: 4.
	byteArray intAt: 1 put: -65535.
	self assert: [ byteArray = #[1 0 255 255] ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:44:36.25'!
testYetAnotherLong
	self assert: [ (#[2 16 1 16 50 84 255 255] longAt: 0+1) = -188900983107582 ]! !

!MMByteArrayExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:46:43.774'!
testYetAnotherLongStore
	| byteArray |
	byteArray := ByteArray new: 8.
	byteArray longAt: 1 put: -188900983107582.
	self assert: [ byteArray = #[2 16 1 16 50 84 255 255] ]! !

TestCase subclass: #MMFloatExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMFloatExtensionTest methodsFor: 'as yet unclassified' stamp: 'WS 6/5/2013 15:21:34.808'!
testDouble

	| value |
	value := Float fromIEEE64Bit: 16r40B3881F9ADD3739.
	self assert: (5000.123456789 - value) abs < Float epsilon.
	! !

TestCase subclass: #MMIntegerExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMIntegerExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:31:35.268'!
testBitInvert32
	| int |
	int := 2r11110000111100001111000011110000.
	self assert: [ int mmBitInvert32 = 2r00001111000011110000111100001111 ].
	
	self assert: [ 2r10101010 mmBitInvert32 =  2r11111111111111111111111101010101].! !

!MMIntegerExtensionTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 14:31:29.792'!
testBitInvert64
	| int |
	int := 2r1111000011110000111100001111000011110000111100001111000011110000.
	self assert: [ int mmBitInvert64 = 2r0000111100001111000011110000111100001111000011110000111100001111 ].
	
	self assert: [ 2r10101010 mmBitInvert64 =  2r1111111111111111111111111111111111111111111111111111111101010101].! !

TestCase subclass: #MMMemoryBuilderTest
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMMemoryBuilderTest methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:53:04.664'!
setUp
	bytes := #[110 166 99 74 251 33 9 64 1 0 255 255 66 255 1 255 2 16 1 16 50 84 255 255 190 186 254 202 190 186 254 202 80 250 0 0 0 0 188 65 1 42 0 0 0 0 0 0]! !

!MMMemoryBuilderTest methodsFor: 'as yet unclassified' stamp: 'WS 7/8/2013 11:45:52.474'!
testExample
	| memory |
	memory := MMMemory builder
		addStruct: {
			#double -> 3.14159258.
			#int -> -16rFFFF.
			#uint -> 16rFF01FF42.
			#long -> -16rABCDEFFEEFFE.
			#ulong -> 16rCAFEBABECAFEBABE.
			#char -> $P asciiValue.
			#uchar -> 250.
			#float -> 23.5.
			#char -> 1.
			#uchar -> 42.
		};
		build.
	self assert: [ memory memory = bytes ]! !

TestCase subclass: #MMMemoryCellTest
	instanceVariableNames: 'memory startAddress size cell address'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMMemoryCellTest methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:42:47.769'!
setUp
	startAddress := 16r1000.
	address := startAddress + 4.
	memory := MMMemory startAddress: startAddress bytes: #(1 2 3 4 5 6 7 8 9 10 11 12).
	
	cell := MMMemoryCell at: address in: memory! !

!MMMemoryCellTest methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 10:42:31.55'!
testAccess
	self assert: [ (cell byteAt: 0) == 5 ].
	self assert: [ (cell byteAt: 1) == 6 ].
	self assert: [ (cell byteAt: 2) == 7 ].
	self assert: [ (cell byteAt: 3) == 8 ].! !

!MMMemoryCellTest methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 10:45:00.558'!
testInitialization
	self assert: [ cell address == address ].
	self assert: [ cell memory == memory ]! !

TestCase subclass: #MMMemoryTest
	instanceVariableNames: 'startAddress size memory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 11:50:56.164'!
expectedFailures
	^ #()! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:43:26.19'!
setUp
	startAddress := 4096.
	size := 4.
	memory := MMMemory startAddress: startAddress cellCount: size! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'WS 5/8/2013 14:09:14.552'!
testByteData
	memory at: startAddress putByteArray: #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16).
	
	1 to: size * 4 do: [ :index |
		self assert: [ (memory byteAt: (index - 1 + startAddress)) == index ]
	]! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'WS 5/8/2013 14:06:38.408'!
testCellAccess
	self shouldnt: [ memory cellAtAddress: startAddress + 4 ] raise: Exception.
	self assert: [ (memory cellAtAddress: startAddress + 4) address == (startAddress + 4) ].
	
	self shouldnt: [ memory cellAtAddress: startAddress + 12 ] raise: Exception.
	self assert: [ (memory cellAtAddress: startAddress + 12) address == (startAddress + 12) ].
	
	self should: [ memory cellAtAddress: startAddress - 4 ] raise: Exception.
	
	self should: [ memory cellAtAddress: startAddress + (size * 4) ] raise: Exception
! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'WS 5/3/2013 15:07:50.83'!
testCellIteration
	| cellIndex |
	cellIndex := 0.
	
	memory cellsDo: [ :cell |
		self assert: [ cell address == (4*cellIndex + startAddress) ].
		cellIndex := cellIndex + 1
	].
	
	self assert: [ cellIndex == size ]! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:42:55.475'!
testEquality
	| secondMemory bytes |
	bytes := #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16).
	memory at: startAddress putByteArray: bytes.
	secondMemory := MMMemory startAddress: startAddress bytes: bytes.
	
	self assert: [ memory = secondMemory ].! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'lt 6/6/2013 13:42:51.444'!
testHashEquality
	| secondMemory bytes |
	bytes := #(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16).
	memory at: startAddress putByteArray: bytes.
	secondMemory := MMMemory startAddress: startAddress bytes: bytes.
	
	self assert: [ memory hash = secondMemory hash ].! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'WS 5/8/2013 14:11:49.834'!
testInitialization
	self assert: [ memory startAddress == startAddress ].
	self assert: [ memory cellCount == size ].
	self assert: [ memory byteSize == (size * MMMemoryCell size)].
	self assert: [ memory endAddress == (startAddress + (4 * size)) ]! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'RL 6/26/2013 11:50:18.676'!
testIntegerData
	memory at: startAddress putInt: 1.
	memory at: startAddress+4 putInt: 2.
	memory at: startAddress+8 putInt: 3.
	memory at: startAddress+12 putInt: 4.
		
	1 to: size do: [ :index |
		self assert: [ (memory intAt: ((index - 1) * 4 + startAddress)) == index ]
	]! !

!MMMemoryTest methodsFor: 'as yet unclassified' stamp: 'RL 5/22/2013 11:15:02.816'!
testLongData
	self shouldnt: [ memory at: startAddress putByteArray: (1 to: 20) asArray ] raise: Exception.
	self assert: [ memory cellCount == size ]! !

TestCase subclass: #MMRightAngledArrowTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMRightAngledArrowTest methodsFor: 'as yet unclassified' stamp: 'WS 5/15/2013 14:05:05.36'!
arrowFrom: aPoint to: anotherPoint shouldHavePoints: aCollection
	self assert: [ (MMRightAngledArrow from: aPoint to: anotherPoint spacing: 30) points asArray = aCollection asArray ]! !

!MMRightAngledArrowTest methodsFor: 'as yet unclassified' stamp: 'WS 5/22/2013 15:55:26.486'!
testPoints
	self arrowFrom: 0@0 to: 0@10 shouldHavePoints: { 0@0. 0@10 }.
	self arrowFrom: 0@0 to: 0@50 shouldHavePoints: { 0@0. 30@0. 30@50. 0@50 }.
	self arrowFrom: 0@0 to: 50@0 shouldHavePoints: { 0@0. 0@30. 50@30. 50@0 }.
	self arrowFrom: 0@0 to: 100@50 shouldHavePoints: { 0@0. 50@0. 50@50. 100@50 }.! !

TestCase subclass: #MMValueMorphTest
	instanceVariableNames: 'valueMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MemoryMentor-Tests'!

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 5/23/2013 15:51:28.901'!
setUp
	valueMorph := MMValueMorph new! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:09.577'!
testAscii
	valueMorph 
		displayMode: #ascii; 
		content: 83.
	self assert: 'S' printString equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:12.535'!
testNegativeBinary
	valueMorph 
		displayMode: #binary; 
		content: -2r10010011.
	self assert: '-10010011b' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:20.761'!
testNegativeDecimal
	valueMorph 
		displayMode: #decimal; 
		content: -33.
	self assert: '-33' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:00.627'!
testNegativeHex
	valueMorph 
		displayMode: #hexadecimal; 
		content: 16r-cafebabe.
	self assert: '-0xCAFEBABE' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:23.084'!
testPositiveBinary
	valueMorph 
		displayMode: #binary; 
		content: 2r10010011.
	self assert: '10010011b' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:25.402'!
testPositiveDecimal
	valueMorph 
		displayMode: #decimal; 
		content: 100.
	self assert: '100' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:27.993'!
testPositiveHex
	valueMorph 
		displayMode: #hexadecimal; 
		content: 16rbabebabe.
	self assert: '0xBABEBABE' equals: valueMorph contentString ! !

!MMValueMorphTest methodsFor: 'as yet unclassified' stamp: 'lt 6/5/2013 15:21:30.382'!
testVeryHighPositiveBinary
	valueMorph 
		displayMode: #binary; 
		content: 2r1111111100000000111111110000000011111111000000001111111100000000.
	self assert: '1111111100000000111111110000000011111111000000001111111100000000b' equals: valueMorph contentString ! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 5/29/2013 11:12:51.548'!
boolAt: index
	^ (self byteAt: index) ~= 0! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 13:00:08.072'!
boolAt: index put: boolean
	self byteAt: index put: (boolean ifTrue: [ 1 ] ifFalse: [ 0 ])! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 13:00:13.892'!
charAt: index
	^ (self byteAt: index) mmFromTwosComplement8! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 7/1/2013 11:53:03.238'!
charAt: index put: aCharacter
	self byteAt: index put: aCharacter value mmToTwosComplement8! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'lt 5/31/2013 11:29:12.65'!
doubleAt: index
	^ Float fromIEEE64Bit: (self ulongAt: index)! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/4/2013 16:30:01.668'!
doubleAt: index put: aDouble
	self ulongAt: index put: aDouble asIEEE64BitDoubleWord! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 5/22/2013 14:12:08.302'!
floatAt: index
	^ Float fromIEEE32Bit: (self uintAt: index)! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 7/8/2013 11:16:48.98'!
floatAt: index put: aFloat
	self uintAt: index put: aFloat asFloat asIEEE32BitWord! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:21:13.33'!
intAt: index
	^ (self uintAt: index) mmFromTwosComplement32! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:21:18.18'!
intAt: index put: anInteger
	self uintAt: index put: anInteger mmToTwosComplement32 ! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 6/26/2013 14:27:13.146'!
longAt: index
	^ (self ulongAt: index) mmFromTwosComplement64! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 13:00:30.56'!
longAt: index put: anInteger
	self ulongAt: index put: anInteger mmToTwosComplement64! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:48:20.038'!
ucharAt: index
	^ self byteAt: index! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 6/27/2013 12:48:04.03'!
ucharAt: index put: aByte
	self byteAt: index put: aByte! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 5/22/2013 12:10:40.646'!
uintAt: index
	| n |
	n := 0.
	3 to: 0 by: -1 do: [ :i |
		n := (n bitShift: 8) + (self byteAt: index + i)
	].
	^ n! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 5/22/2013 17:21:27.09'!
uintAt: index put: anInteger
	3 to: 0 by: -1 do: [ :i |
		self byteAt: index + i put: (anInteger digitAt: i + 1).
	]! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'RL 5/22/2013 13:23:39.95'!
ulongAt: index
	| n |
	n := 0.
	7 to: 0 by: -1 do: [ :i |
		n := (n bitShift: 8) + (self byteAt: index + i)
	].
	^ n! !

!ByteArray methodsFor: '*MemoryMentor-Core' stamp: 'WS 5/22/2013 17:21:11.238'!
ulongAt: index put: anInteger
	7 to: 0 by: -1 do: [ :i |
		self byteAt: index + i put: (anInteger digitAt: i+1).
	]! !
